# -*- coding: UTF-8 -*-
"""
Name: filter_stab.py
Porpose: Show dialog to get vidstab data based on FFmpeg syntax
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.23.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import webbrowser
import wx
import wx.lib.agw.floatspin as FS
from videomass.vdms_utils.utils import time_to_integer
from videomass.vdms_utils.utils import integer_to_time
from videomass.vdms_utils.utils import clockset
from videomass.vdms_io import io_tools
from videomass.vdms_threads.generic_task import FFmpegGenericTask
from videomass.vdms_dialogs.widget_utils import PopupDialog
from videomass.vdms_io.make_filelog import make_log_template


class VidstabSet(wx.Dialog):
    """
    A dialog tool to get vidstabdetect and vidstabtransform
    data based on FFmpeg syntax.

    """
    get = wx.GetApp()
    appdata = get.appset
    LOGDIR = appdata['logdir']
    TMPROOT = os.path.join(appdata['cachedir'], 'tmp', 'VidStab')
    TMPSRC = os.path.join(TMPROOT, 'tmpedit')
    os.makedirs(TMPSRC, mode=0o777, exist_ok=True)

    def __init__(self, parent, *args, **kwa):
        """
        parameters defined here:
        vidstabdetect parameters for pass one.
        vidstabtransform parameters for pass two
        unsharp parameters for pass two
        makeduo, to produce another video for comparison

        """
        self.vidstabdetect = args[0]
        self.vidstabtransform = args[1]
        self.unsharp = args[2]
        self.makeduo = args[3]

        self.filename = kwa['filename']
        name = os.path.splitext(os.path.basename(self.filename))[0]
        self.framesrc = os.path.join(VidstabSet.TMPSRC, f'{name}.mkv')
        self.frameduo = os.path.join(VidstabSet.TMPSRC, f'{name}_DUO.mkv')
        self.fileclock = os.path.join(VidstabSet.TMPROOT, f'{name}.clock')
        tcheck = clockset(kwa['duration'], self.fileclock)
        self.clock = tcheck['duration']
        self.mills = tcheck['millis']
        self.logfile = None
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizerBase = wx.BoxSizer(wx.VERTICAL)
        boxenable = wx.BoxSizer(wx.HORIZONTAL)
        sizerBase.Add(boxenable, 0, wx.CENTER, 5)
        self.ckbx_enable = wx.CheckBox(self, wx.ID_ANY,
                                       _('Enable stabilizer'))
        boxenable.Add(self.ckbx_enable, 0, wx.ALL | wx.CENTER, 2)
        self.ckbx_duo = wx.CheckBox(self, wx.ID_ANY, _('Create a side-by-side '
                                                       'comparison video'))
        boxenable.Add(self.ckbx_duo, 0, wx.ALL | wx.CENTER, 2)
        # preview
        sizerBase.Add((5, 5), 0)
        msg = _("Create a snapshot")
        stboxtime = wx.StaticBox(self, wx.ID_ANY, msg)
        sizertime = wx.StaticBoxSizer(stboxtime, wx.HORIZONTAL)
        sizerBase.Add(sizertime, 0, wx.EXPAND | wx.ALL, 5)

        boxtime = wx.BoxSizer(wx.HORIZONTAL)
        sizertime.Add(boxtime, 0, wx.ALL | wx.CENTER, 5)

        self.sld_time = wx.Slider(self, wx.ID_ANY,
                                  time_to_integer(self.clock),
                                  0,
                                  1 if not self.mills else self.mills,
                                  size=(250, -1),
                                  style=wx.SL_HORIZONTAL,
                                  )
        boxtime.Add(self.sld_time, 0, wx.CENTER, 0)
        self.lab_time = wx.StaticText(self, wx.ID_ANY, self.clock)
        boxtime.Add(self.lab_time, 0, wx.LEFT | wx.CENTER, 20)
        self.btn_snap = wx.Button(self, wx.ID_ANY, _("Preview"))
        self.btn_snap.Disable()
        boxtime.Add(self.btn_snap, 0, wx.LEFT | wx.CENTER, 20)
        self.lab_dur = wx.StaticText(self, wx.ID_ANY, _("Duration seconds:"))
        boxtime.Add(self.lab_dur, 0, wx.LEFT | wx.CENTER, 20)
        self.spin_dur = wx.SpinCtrl(self, wx.ID_ANY, "5",
                                    min=3, max=15,
                                    style=wx.TE_PROCESS_ENTER
                                    | wx.SP_ARROW_KEYS,
                                    )
        boxtime.Add(self.spin_dur, 0, wx.LEFT | wx.CENTER, 2)
        # Box detect
        sbox = wx.StaticBox(self, wx.ID_ANY, (_("Video Detect")))
        box_detect = wx.StaticBoxSizer(sbox, wx.VERTICAL)
        sizerBase.Add(box_detect, 0, wx.ALL | wx.EXPAND, 5)
        Flex_detect1 = wx.FlexGridSizer(1, 6, 0, 0)
        box_detect.Add(Flex_detect1, 0, wx.ALL, 5)
        self.lab_shake = wx.StaticText(self, wx.ID_ANY, ("Shakiness:"))
        Flex_detect1.Add(self.lab_shake, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 2
                         )
        self.spin_shake = wx.SpinCtrl(self, wx.ID_ANY, "5",
                                      min=1, max=10,
                                      style=wx.TE_PROCESS_ENTER
                                      | wx.SP_ARROW_KEYS
                                      )
        Flex_detect1.Add(self.spin_shake, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 15
                         )
        self.lab_accuracy = wx.StaticText(self, wx.ID_ANY, ("Accuracy:"))
        Flex_detect1.Add(self.lab_accuracy, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 2
                         )
        self.spin_accuracy = wx.SpinCtrl(self, wx.ID_ANY, "15",
                                         min=1, max=15,
                                         style=wx.TE_PROCESS_ENTER
                                         | wx.SP_ARROW_KEYS
                                         )
        Flex_detect1.Add(self.spin_accuracy, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 15
                         )
        self.lab_stepsize = wx.StaticText(self, wx.ID_ANY, ("Stepsize:"))
        Flex_detect1.Add(self.lab_stepsize, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 2
                         )
        self.spin_stepsize = wx.SpinCtrl(self, wx.ID_ANY, "6",
                                         min=1, max=10000,
                                         style=wx.TE_PROCESS_ENTER
                                         | wx.SP_ARROW_KEYS
                                         )
        Flex_detect1.Add(self.spin_stepsize, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 15
                         )
        Flex_detect2 = wx.FlexGridSizer(1, 4, 0, 0)
        box_detect.Add(Flex_detect2, 0, wx.ALL, 5)
        self.lab_mincontr = wx.StaticText(self, wx.ID_ANY, ("Mincontrast:"))
        Flex_detect2.Add(self.lab_mincontr, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 2)
        self.spin_mincontr = FS.FloatSpin(self, wx.ID_ANY,
                                          min_val=0, max_val=1,
                                          increment=0.01, value=0.25,
                                          agwStyle=FS.FS_LEFT,
                                          size=(120, -1),
                                          )
        self.spin_mincontr.SetFormat("%f")
        self.spin_mincontr.SetDigits(2)
        Flex_detect2.Add(self.spin_mincontr, 0, wx.RIGHT
                         | wx.ALIGN_CENTER_VERTICAL, 15
                         )
        Flex_detect2.Add((15, 0), 0, wx.ALL, 5)
        lab = _("[TRIPOD] Enable tripod mode if checked")
        self.ckbx_tripod1 = wx.CheckBox(self, wx.ID_ANY, lab)
        Flex_detect2.Add(self.ckbx_tripod1, 0, wx.ALL
                         | wx.ALIGN_CENTER_VERTICAL, 5
                         )
        # Box transform
        sizerBase.Add((15, 0), 0, wx.ALL, 5)
        sbox = wx.StaticBox(self, wx.ID_ANY, (_("Video transform")))
        box_trans = wx.StaticBoxSizer(sbox, wx.VERTICAL)
        sizerBase.Add(box_trans, 1, wx.ALL | wx.EXPAND, 5)
        flex_trans1 = wx.FlexGridSizer(1, 5, 0, 0)
        box_trans.Add(flex_trans1, 0, wx.ALL, 5)
        self.lab_smooth = wx.StaticText(self, wx.ID_ANY, ("Smoothing:"))
        flex_trans1.Add(self.lab_smooth, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.spin_smooth = wx.SpinCtrl(self, wx.ID_ANY, "15",
                                       min=0, max=30,
                                       style=wx.TE_PROCESS_ENTER
                                       | wx.SP_ARROW_KEYS,
                                       size=(-1, -1),
                                       )
        flex_trans1.Add(self.spin_smooth, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 15
                        )
        self.rdb_optalgo = wx.RadioBox(self, wx.ID_ANY,
                                       (_("[OPTALGO] optimization algorithm")),
                                       choices=[("gauss"), ("avg")],
                                       majorDimension=1,
                                       style=wx.RA_SPECIFY_ROWS,
                                       )
        flex_trans1.Add(self.rdb_optalgo, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 15
                        )
        self.lab_maxangle = wx.StaticText(self, wx.ID_ANY, ("Maxangle:"))
        flex_trans1.Add(self.lab_maxangle, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.spin_maxangle = wx.SpinCtrl(self, wx.ID_ANY, "-1",
                                         min=-1, max=360,
                                         style=wx.TE_PROCESS_ENTER
                                         | wx.SP_ARROW_KEYS,
                                         size=(-1, -1),
                                         )
        flex_trans1.Add(self.spin_maxangle, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 5
                        )
        self.rdb_crop = wx.RadioBox(self, wx.ID_ANY,
                                    (_("[CROP] Specify how to deal with "
                                        "borders at movement compensation")),
                                    choices=[("keep"), ("black")],
                                    majorDimension=1,
                                    style=wx.RA_SPECIFY_ROWS,
                                    )
        box_trans.Add(self.rdb_crop, 0, wx.ALL, 5)
        lab = _("[INVERT] Invert transforms if checked")
        self.ckbx_invert = wx.CheckBox(self, wx.ID_ANY, lab)
        box_trans.Add(self.ckbx_invert, 0, wx.ALL, 5)
        lab = _("[RELATIVE] Consider transforms as relative to previous "
                "frame if checked, absolute if unchecked")
        self.ckbx_relative = wx.CheckBox(self, wx.ID_ANY, lab)
        box_trans.Add(self.ckbx_relative, 0, wx.ALL, 5)
        flex_trans2 = wx.FlexGridSizer(1, 6, 0, 0)
        box_trans.Add(flex_trans2, 0, wx.ALL, 5)
        self.lab_zoom = wx.StaticText(self, wx.ID_ANY, ("Zoom:"))
        flex_trans2.Add(self.lab_zoom, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.spin_zoom = wx.SpinCtrl(self, wx.ID_ANY, "0",
                                     min=0, max=100,
                                     style=wx.TE_PROCESS_ENTER
                                     | wx.SP_ARROW_KEYS,
                                     size=(-1, -1),
                                     )
        flex_trans2.Add(self.spin_zoom, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 15
                        )
        self.lab_optzoom = wx.StaticText(self, wx.ID_ANY, ("OptZoom:"))
        flex_trans2.Add(self.lab_optzoom, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.spin_optzoom = wx.SpinCtrl(self, wx.ID_ANY, "1",
                                        min=0, max=2,
                                        style=wx.TE_PROCESS_ENTER
                                        | wx.SP_ARROW_KEYS,
                                        size=(-1, -1),
                                        )
        flex_trans2.Add(self.spin_optzoom, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 15
                        )
        self.lab_zoomspeed = wx.StaticText(self, wx.ID_ANY, ("Zoom speed:"))
        flex_trans2.Add(self.lab_zoomspeed, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.spin_zoomspeed = FS.FloatSpin(self, wx.ID_ANY,
                                           min_val=0, max_val=5,
                                           increment=0.05, value=0.25,
                                           agwStyle=FS.FS_LEFT, size=(120, -1)
                                           )
        self.spin_zoomspeed.SetFormat("%f")
        self.spin_zoomspeed.SetDigits(2)
        flex_trans2.Add(self.spin_zoomspeed, 0, wx.RIGHT, 5)
        self.rdb_interpol = wx.RadioBox(self, wx.ID_ANY,
                                        (_("Specify type of interpolation")),
                                        choices=[("no"), ("linear"),
                                                 ("bilinear"), ("bicubic")],
                                        majorDimension=1,
                                        style=wx.RA_SPECIFY_ROWS,
                                        )
        self.rdb_interpol.SetSelection(2)
        box_trans.Add(self.rdb_interpol, 0, wx.ALL, 5)
        lab = _("[TRIPOD2] virtual tripod mode, equivalent to "
                "relative=unchecked:smoothing=0")
        self.ckbx_tripod2 = wx.CheckBox(self, wx.ID_ANY, lab)
        box_trans.Add(self.ckbx_tripod2, 0, wx.ALL, 5)
        flex_trans3 = wx.FlexGridSizer(1, 2, 0, 0)
        box_trans.Add(flex_trans3, 0, wx.ALL, 5)
        self.lab_unsharp = wx.StaticText(self, wx.ID_ANY,
                                         (_("Unsharp filter:")))
        flex_trans3.Add(self.lab_unsharp, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 2
                        )
        self.txt_unsharp = wx.TextCtrl(self, wx.ID_ANY,
                                       value="unsharp=5:5:0.8:3:3:0.4",
                                       size=(500, -1),
                                       )
        flex_trans3.Add(self.txt_unsharp, 0, wx.RIGHT
                        | wx.ALIGN_CENTER_VERTICAL, 5
                        )
        # ----- confirm buttons section
        gridbtns = wx.GridSizer(1, 2, 0, 0)
        gridhelp = wx.GridSizer(1, 1, 0, 0)
        btn_help = wx.Button(self, wx.ID_HELP, "")
        gridhelp.Add(btn_help, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        gridbtns.Add(gridhelp)
        boxaff = wx.BoxSizer(wx.HORIZONTAL)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        boxaff.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        boxaff.Add(btn_ok, 0, wx.LEFT, 5)
        btn_reset = wx.Button(self, wx.ID_ANY, _("Reset"))
        btn_reset.SetBitmap(args[4], wx.LEFT)
        boxaff.Add(btn_reset, 0, wx.LEFT, 5)
        gridbtns.Add(boxaff, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizerBase.Add(gridbtns, 0, wx.EXPAND)

        # ----- Set properties
        tip = _('Seek to a position on the video.')
        self.sld_time.SetToolTip(tip)
        tip = _('Make a snapshot of the video with the settings made.')
        self.btn_snap.SetToolTip(tip)
        tip = _('Set the snapshot duration from 3 to 15 seconds from '
                'the seek position, default is 5 seconds.')
        self.spin_dur.SetToolTip(tip)
        # vidstabdetect
        tip = _('Set how shaky the video is and how quick the camera is. A '
                'value of 1 means little shakiness, a value of 10 means '
                'strong shakiness. Default value is 5.')
        self.spin_shake.SetToolTip(tip)
        tip = _('Set the accuracy of the detection process. A value of 1 '
                'means low accuracy, a value of 15 means high accuracy. '
                'Default value is 15.')
        self.spin_accuracy.SetToolTip(tip)
        tip = _('Set stepsize of the search process. The region around '
                'minimum is scanned with 1 pixel resolution. Default value '
                'is 6.')
        self.spin_stepsize.SetToolTip(tip)
        tip = _('Set minimum contrast. Below this value a local measurement '
                'field is discarded. The range is 0-1. Default value is 0.25.')
        self.spin_mincontr.SetToolTip(tip)
        tip = _('Set reference frame number for tripod mode. If enabled, the '
                'motion of the frames is compared to a reference frame in '
                'the filtered stream, identified by the specified number. '
                'The idea is to compensate all movements in a more-or-less '
                'static scene and keep the camera view absolutely still. '
                'The frames are counted starting from 1.')
        self.ckbx_tripod1.SetToolTip(tip)

        # vidstabtransform
        tip = _('Set the number of frames (value*2 + 1) used for lowpass '
                'filtering the camera movements. Default value is 15. For '
                'example a number of 10 means that 21 frames are used (10 in '
                'the past and 10 in the future) to smoothen the motion in '
                'the video. A larger value leads to a smoother video, but '
                'limits the acceleration of the camera (pan/tilt movements). '
                '0 is a special case where a static camera is simulated.')
        self.spin_smooth.SetToolTip(tip)
        tip = _('Set the camera path optimization algorithm. Values are: '
                '`gauss`: gaussian kernel low-pass filter on camera motion '
                '(default), and `avg`: averaging on transformations')
        self.rdb_optalgo.SetToolTip(tip)
        tip = _('Set maximal angle in radians (degree*PI/180) to rotate '
                'frames. Default value is -1, meaning no limit.')
        self.spin_maxangle.SetToolTip(tip)
        tip = _('Specify how to deal with borders that may be visible due to '
                'movement compensation. Values are: `keep` keep image '
                'information from previous frame (default), `black` fill the '
                'border black')
        self.rdb_crop.SetToolTip(tip)
        tip = _('Invert transforms if checked. Default is unchecked.')
        self.ckbx_invert.SetToolTip(tip)
        tip = _('Consider transforms as relative to previous frame if '
                'checked, absolute if unchecked. Default is checked.')
        self.ckbx_relative.SetToolTip(tip)
        tip = _('Set percentage to zoom. A positive value will result in a '
                'zoom-in effect, a negative value in a zoom-out effect. '
                'Default value is 0 (no zoom).')
        self.spin_zoom.SetToolTip(tip)
        tip = _('Set optimal zooming to avoid borders. Accepted values are: '
                '`0` disabled, `1` optimal static zoom value is determined '
                '(only very strong movements will lead to visible borders) '
                '(default), `2` optimal adaptive zoom value is determined '
                '(no borders will be visible), see zoomspeed. Note that the '
                'value given at zoom is added to the one calculated here.')
        self.spin_optzoom.SetToolTip(tip)
        tip = _('Set percent to zoom maximally each frame (enabled when '
                'optzoom is set to 2). Range is from 0 to 5, default value '
                'is 0.25.')
        self.spin_zoomspeed.SetToolTip(tip)
        tip = _('Specify type of interpolation. Available values are: `no` '
                'no interpolation, `linear` linear only horizontal, '
                '`bilinear` linear in both directions (default), `bicubic` '
                'cubic in both directions (slow)')
        self.rdb_interpol.SetToolTip(tip)
        tip = _('Enable virtual tripod mode if checked, which is equivalent '
                'to relative=0:smoothing=0. Default is unchecked. Use also '
                'tripod option of vidstabdetect.')
        self.ckbx_tripod2.SetToolTip(tip)
        # unsharp filter
        tip = _('Sharpen or blur the input video. Note the use of the '
                'unsharp filter which is always recommended.')
        self.txt_unsharp.SetToolTip(tip)

        self.SetTitle(_("Video Stabilizer Tool"))

        # ----- Set layout
        self.SetSizer(sizerBase)
        sizerBase.Fit(self)
        self.Layout()

        # ----------------------Binding (EVT)--------------------------#
        self.Bind(wx.EVT_CHECKBOX, self.on_activate, self.ckbx_enable)
        self.Bind(wx.EVT_COMMAND_SCROLL, self.on_seek_time, self.sld_time)
        self.Bind(wx.EVT_BUTTON, self.on_load_at_time, self.btn_snap)
        self.Bind(wx.EVT_SPINCTRL, self.on_optzoom, self.spin_optzoom)
        self.Bind(wx.EVT_CHECKBOX, self.on_Tripod1, self.ckbx_tripod1)
        self.Bind(wx.EVT_CHECKBOX, self.on_Tripod2, self.ckbx_tripod2)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)
        self.Bind(wx.EVT_BUTTON, self.set_default, btn_reset)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)

        if self.vidstabdetect:
            self.set_values()  # Set previous changes
        else:
            self.set_default(self)
            self.on_activate(self)
    # ------------------------------------------------------------------#

    def concat_filter(self, orderf):
        """
        Concatenates all VidstabSet values.
        Returns the VidstabSet filter string in ffmpeg syntax.
        """
        concat = ''.join([f'{x},' for x in orderf if x])[:-1]
        if not concat:
            return concat
        return f'-vf {concat}'
    # -----------------------------------------------------------------------#

    def on_seek_time(self, event):
        """
        Event to get slider position.
        Getting the value in ms from the time slider,
        converts it to clock format e.g (00:00:00),
        and sets the label with the converted value.
        """
        seek = self.sld_time.GetValue()
        clock = integer_to_time(seek, False)  # to 24-hour
        self.lab_time.SetLabel(clock)  # update StaticText
        if not self.btn_snap.IsEnabled():
            self.btn_snap.Enable()
    # ------------------------------------------------------------------#

    def on_load_at_time(self, event):
        """
        Reloads processes at a given time clock point
        """
        data = self.getvalue()
        detect = f'-vf {data[0]}'
        trasform = self.concat_filter((data[1], data[2]))
        self.logfile = make_log_template('generic_task.log',
                                         VidstabSet.LOGDIR,
                                         mode="w",
                                         )
        error = self.process(self.filename, args=detect, mode='detect')
        if error:
            wx.MessageBox(f'{error}', _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        error = self.process(self.filename,
                             self.framesrc,
                             args=trasform,
                             mode='trasform',
                             )
        if error:
            wx.MessageBox(f'{error}', _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        if self.ckbx_duo.IsChecked():
            error = self.process(self.filename,
                                 self.frameduo,
                                 args='',
                                 mode='makeduo',
                                 )
            if error:
                wx.MessageBox(f'{error}', _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
                return
            io_tools.openpath(self.frameduo)
            return

        io_tools.openpath(self.framesrc)
        return
    # ------------------------------------------------------------------#

    def process(self, infile, outfile=None, args='', mode=None):
        """
        Generate a new frame at the clock position using
        ffmpeg `eq` filter.
        """
        if not self.mills:
            sseg, tseg = '', ''
        else:
            seek = self.sld_time.GetValue()
            stime = self.spin_dur.GetValue() * 1000
            if seek + stime > self.mills:
                seek = self.mills - stime
                if seek < 0:
                    seek, stime = 0, self.mills
            duration = integer_to_time(stime, False)  # to 24-hour
            self.clock = integer_to_time(seek, False)  # to 24-hour
            sseg = f'-ss {self.clock}.000'
            tseg = f'-t {duration}.000'

        if mode == 'detect':
            nul = ('NUL' if VidstabSet.appdata['ostype']
                   == 'Windows' else '/dev/null')
            argstr = f'{sseg} -i "{infile}" {tseg} {args} -f null {nul}'

        elif mode == 'trasform':
            argstr = f'{sseg} -i "{infile}" {tseg} {args} "{outfile}"'

        elif mode == 'makeduo':
            argstr = (f'{sseg} -i "{infile}" {tseg} -i "{self.framesrc}" '
                      f'{tseg} -filter_complex hstack "{outfile}"')
        else:
            return None

        thread = FFmpegGenericTask(argstr,
                                   procname=f'VidStab - {mode}',
                                   logfile=self.logfile,
                                   )
        caption = _("Videomass - Loading...")
        msg = _("Please wait,\nThis process will take a few seconds.")
        dlgload = PopupDialog(self, caption, msg, thread)
        dlgload.ShowModal()
        thread.join()  # wait end thread
        error = thread.status
        if error:
            dlgload.Destroy()
            return error
        dlgload.Destroy()
        return None
    # ------------------------------------------------------------------#

    def set_default(self, event):
        """
        Revert all control values to default

        """
        # vidstabdetect
        self.spin_shake.SetValue('5')
        self.spin_accuracy.SetValue('15')
        self.spin_stepsize.SetValue('6')
        self.spin_mincontr.SetValue(0.25)
        self.ckbx_tripod1.SetValue(False)
        # vidstabtransform
        self.spin_smooth.SetValue('15')
        self.rdb_optalgo.SetSelection(0)
        self.spin_maxangle.SetValue('-1')
        self.rdb_crop.SetSelection(0)
        self.ckbx_invert.SetValue(False)
        self.ckbx_relative.SetValue(True)
        self.spin_zoom.SetValue('0')
        self.spin_optzoom.SetValue('1')
        self.spin_zoomspeed.SetValue(0.25)
        self.spin_zoomspeed.Disable()
        self.rdb_interpol.SetSelection(2)
        self.ckbx_tripod2.SetValue(False)
        self.ckbx_tripod2.Disable()
        # unsharp filter
        self.txt_unsharp.Clear()
        self.txt_unsharp.write('unsharp=5:5:0.8:3:3:0.4')
    # ------------------------------------------------------------------#

    def set_values(self):
        """
        set values previously confirmed

        """
        k_v_detect = dict(i.split('=') for i in self.vidstabdetect.split(
                          'vidstabdetect=')[1].split(':'))

        self.spin_shake.SetValue(k_v_detect['shakiness'])
        self.spin_accuracy.SetValue(k_v_detect['accuracy'])
        self.spin_stepsize.SetValue(k_v_detect['stepsize'])
        self.spin_mincontr.SetValue(float(k_v_detect['mincontrast']))
        if k_v_detect['tripod'] == '1':
            self.ckbx_tripod1.SetValue(True)
            self.ckbx_tripod2.SetValue(True)
            self.ckbx_tripod2.Enable()
        else:
            self.ckbx_tripod1.SetValue(False)
            self.ckbx_tripod2.SetValue(False)
            self.ckbx_tripod2.Disable()

        k_v_transf = dict(i.split('=') for i in self.vidstabtransform.split(
                          'vidstabtransform=')[1].split(':'))

        self.spin_smooth.SetValue(k_v_transf['smoothing'])
        if k_v_transf['optalgo'] == 'gauss':
            self.rdb_optalgo.SetSelection(0)
        else:
            self.rdb_optalgo.SetSelection(1)
        self.spin_maxangle.SetValue(k_v_transf['maxangle'])
        if k_v_transf['crop'] == 'keep':
            self.rdb_crop.SetSelection(0)
        else:
            self.rdb_crop.SetSelection(1)
        if k_v_transf['invert'] == '1':
            self.ckbx_invert.SetValue(True)
        else:
            self.ckbx_invert.SetValue(False)
        if k_v_transf['relative'] == '1':
            self.ckbx_relative.SetValue(True)
        else:
            self.ckbx_relative.SetValue(False)
        self.spin_zoom.SetValue(k_v_transf['zoom'])
        self.spin_optzoom.SetValue(k_v_transf['optzoom'])
        if k_v_transf['optzoom'] == '2':
            self.spin_zoomspeed.Enable()
            self.lab_zoomspeed.Enable()
        else:
            self.spin_zoomspeed.Disable()
            self.lab_zoomspeed.Disable()
        self.spin_zoomspeed.SetValue(float(k_v_transf['zoomspeed']))
        if k_v_transf['interpol'] == 'no':
            self.rdb_interpol.SetSelection(0)
        elif k_v_transf['interpol'] == 'linear':
            self.rdb_interpol.SetSelection(1)
        elif k_v_transf['interpol'] == 'bilinear':
            self.rdb_interpol.SetSelection(2)
        else:
            self.rdb_interpol.SetSelection(3)

        self.txt_unsharp.Clear()
        self.txt_unsharp.write(self.unsharp)
        self.ckbx_enable.SetValue(True)
        self.ckbx_duo.SetValue(self.makeduo)
        self.on_activate(self)

    # ----------------------Event handler (callback)---------------------#

    def on_optzoom(self, event):
        """
        If optzoom value is equal to 2, enable zoomspeed

        """
        if self.spin_optzoom.GetValue() == 2:
            self.spin_zoomspeed.Enable()
            self.lab_zoomspeed.Enable()
        else:
            self.spin_zoomspeed.SetValue(0.25)
            self.spin_zoomspeed.Disable()
            self.lab_zoomspeed.Disable()
    # ------------------------------------------------------------------#

    def on_Tripod1(self, event):
        """
        If self.ckbx_tripod1 is checked, enable and
        check self.ckbx_tripod2

        """
        if self.ckbx_tripod1.IsChecked():
            self.ckbx_tripod2.Enable()
            self.ckbx_tripod2.SetValue(True)
        else:
            self.ckbx_tripod2.Disable()
            self.ckbx_tripod2.SetValue(False)
    # ------------------------------------------------------------------#

    def on_Tripod2(self, event):
        """
        If self.ckbx_tripod2 is unchecked, disable it and
        uncheck self.ckbx_tripod1

        """
        if not self.ckbx_tripod2.IsChecked():
            self.ckbx_tripod1.SetValue(False)
            self.on_Tripod1(self)

    # ------------------------------------------------------------------#

    def on_help(self, event):
        """
        Open default web browser via Python Web-browser controller.
        see <https://docs.python.org/3.8/library/webbrowser.html>

        """
        page = ('https://jeanslack.github.io/Videomass/User-guide'
                '/Video_filters_en.pdf')

        webbrowser.open(page)
    # ------------------------------------------------------------------#

    def on_activate(self, event):
        """
        Enable or disable vidstab filter

        """
        if self.ckbx_enable.IsChecked():
            self.spin_shake.Enable()
            self.spin_accuracy.Enable()
            self.spin_stepsize.Enable()
            self.spin_mincontr.Enable()
            self.ckbx_tripod1.Enable()
            self.spin_smooth.Enable()
            self.rdb_optalgo.Enable()
            self.spin_maxangle.Enable()
            self.rdb_crop.Enable()
            self.ckbx_invert.Enable()
            self.ckbx_relative.Enable()
            self.spin_zoom.Enable()
            self.spin_optzoom.Enable()
            self.lab_shake.Enable()
            self.lab_accuracy.Enable()
            self.lab_stepsize.Enable()
            self.lab_mincontr.Enable()
            self.lab_smooth.Enable()
            self.lab_maxangle.Enable()
            self.lab_zoom.Enable()
            self.lab_optzoom.Enable()
            self.lab_unsharp.Enable()
            # self.spin_zoomspeed.Enable(), self.lab_zoomspeed.Enable()
            self.rdb_interpol.Enable()
            # self.ckbx_tripod2.Enable()
            self.txt_unsharp.Enable()
            self.ckbx_duo.Enable()
            # enable preview only if greater than 3 seconds
            if self.mills > 3000:
                self.btn_snap.Enable()
                self.sld_time.Enable()
                self.lab_time.Enable()
                self.lab_dur.Enable()
                self.spin_dur.Enable()
        else:
            self.spin_shake.Disable()
            self.spin_accuracy.Disable()
            self.spin_stepsize.Disable()
            self.spin_mincontr.Disable()
            self.ckbx_tripod1.Disable()
            self.spin_smooth.Disable()
            self.rdb_optalgo.Disable()
            self.spin_maxangle.Disable()
            self.rdb_crop.Disable()
            self.ckbx_invert.Disable()
            self.ckbx_relative.Disable()
            self.spin_zoom.Disable()
            self.spin_optzoom.Disable()
            self.spin_zoomspeed.Disable()
            self.rdb_interpol.Disable()
            self.ckbx_tripod2.Disable()
            self.txt_unsharp.Disable()
            self.lab_shake.Disable()
            self.lab_accuracy.Disable()
            self.lab_stepsize.Disable()
            self.lab_mincontr.Disable()
            self.lab_smooth.Disable()
            self.lab_maxangle.Disable()
            self.lab_zoom.Disable()
            self.lab_optzoom.Disable()
            self.lab_zoomspeed.Disable()
            self.lab_unsharp.Disable()
            # disable makeduo
            self.ckbx_duo.SetValue(False)
            self.ckbx_duo.Disable()
            # disable preview
            self.btn_snap.Disable()
            self.sld_time.Disable()
            self.lab_time.Disable()
            self.lab_dur.Disable()
            self.spin_dur.Disable()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Don't use self.Destroy() in this dialog
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See the caller for more info and usage.
        """
        if not self.ckbx_enable.IsChecked():
            return None

        # vidstabdetect
        shakiness = self.spin_shake.GetValue()
        accuracy = self.spin_accuracy.GetValue()
        stepsize = self.spin_stepsize.GetValue()
        mincontrast = self.spin_mincontr.GetValue()
        if self.ckbx_tripod1.IsChecked():
            tripod1, tripod2 = '1', '1'
        else:
            tripod1, tripod2 = '0', '0'
        # vidstabtransform
        smoothing = self.spin_smooth.GetValue()
        optalgo = self.rdb_optalgo.GetString(self.rdb_optalgo.GetSelection())
        maxangle = self.spin_maxangle.GetValue()
        crop = self.rdb_crop.GetString(self.rdb_crop.GetSelection())

        invert = '1' if self.ckbx_invert.IsChecked() else '0'
        relative = '1' if self.ckbx_relative.IsChecked() else '0'

        zoom = self.spin_zoom.GetValue()
        optzoom = self.spin_optzoom.GetValue()
        zoomspeed = self.spin_zoomspeed.GetValue()
        interp = self.rdb_interpol.GetString(self.rdb_interpol.GetSelection())

        vidstabdetect = (f'vidstabdetect=shakiness={shakiness}:'
                         f'accuracy={accuracy}:'
                         f'stepsize={stepsize}:'
                         f'mincontrast={mincontrast}:'
                         f'tripod={tripod1}:'
                         f'show=0'
                         )
        vidstabtransform = (f'vidstabtransform=smoothing={smoothing}:'
                            f'optalgo={optalgo}:'
                            f'maxshift=-1:'
                            f'maxangle={maxangle}:'
                            f'crop={crop}:'
                            f'invert={invert}:'
                            f'relative={relative}:'
                            f'zoom={zoom}:'
                            f'optzoom={optzoom}:'
                            f'zoomspeed={zoomspeed}:'
                            f'interpol={interp}:'
                            f'tripod={tripod2}'
                            )
        unsharp = self.txt_unsharp.GetValue()
        makeduo = self.ckbx_duo.IsChecked()

        return vidstabdetect, vidstabtransform, unsharp, makeduo



# -*- coding: UTF-8 -*-
"""
Name: infoprog.py
Porpose: about videomass
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: March.13.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import wx
import wx.adv
from videomass.vdms_sys.msg_info import current_release
from videomass.vdms_sys.msg_info import descriptions_release


def aboutdlg(parent, videomass_icon):
    """
    It's a predefined template to create a dialogue on
    the program information

    """
    # ------------------
    cr = current_release()
    name_upper = cr[0]
    # name_lower = cr[1]
    version = cr[2]
    # Release = cr[3]
    copyr = cr[4]
    website = cr[5]
    author = cr[6]
    mail = cr[7]
    # comment = cr[8]
    # -----------------
    dr = descriptions_release()
    # Short_Dscrp = dr[0]
    # Long_Dscrp = dr[1]
    # Short_Lic = dr[2]
    long_lic = dr[3]
    # ------------------
    info = wx.adv.AboutDialogInfo()
    info.SetIcon(wx.Icon(videomass_icon, type=wx.BITMAP_TYPE_PNG))
    info.SetName(f"{name_upper}")
    info.SetVersion(f"v{version}")
    info.SetDescription(_("Cross-platform graphical user interface for FFmpeg "
                          "and yt-dlp.\n"))
    info.SetCopyright(f"Copyright {copyr} {author[0]} {author[1]}")
    info.SetWebSite(website)
    info.SetLicence(long_lic)
    info.AddDeveloper(f"{author[0]} <{mail}>")
    info.AddDocWriter("Gianluca Pernigotto <jeanlucperni@gmail.com>")
    info.AddTranslator("Gianluca Pernigotto <jeanlucperni@gmail.com> (it_IT)")
    info.AddTranslator("ChourS <ChourS2008@yandex.ru> (ru_RU)")
    info.AddTranslator("Roelof Berkepeis <roelof@imoma.eu> (nl_NL)")
    info.AddTranslator("johannesdedoper https://github.com/johannesdedoper "
                       "(nl_NL)")
    info.AddTranslator("Samuel (pt_BR)")
    info.AddTranslator("katnatek from blogdrake https://www.blogdrake.net/ "
                       "(es_ES)")
    info.AddTranslator("Phil Aug <philiaug@live.fr> (fr_FR)")
    info.AddTranslator("MaiJZ https://github.com/maijz128 (zh_CN)")

    info.SetArtists(['WxPython Phoenix <wxpython-users@googlegroups.com>',
                     'Gianluca Pernigotto <jeanlucperni@gmail.com>'])
    wx.adv.AboutBox(info)
    # event.Skip()



# -*- coding: UTF-8 -*-
"""
Name: check_bin.py
Porpose: Gets the output to display the features of FFmpeg
Compatibility: Python3 (Unix, Windows)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.15.2021
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import subprocess
from videomass.vdms_utils.utils import Popen


def subp(args, ostype):
    """
    Execute commands which *do not* need to read the stdout/stderr in
    real time.

    Parameters:
        [*args* command list object]
        *ostype* result of the platform.system()

    """
    cmd = []
    for opt in args:
        cmd.append(opt)

    if ostype == 'Windows':
        cmd = ' '.join(cmd)
    try:
        with Popen(cmd,
                   stdout=subprocess.PIPE,
                   stderr=subprocess.STDOUT,
                   universal_newlines=True,  # mod text
                   encoding='utf-8',
                   ) as proc:
            out = proc.communicate()

            if proc.returncode:  # if returncode == 1
                return ('Not found', out[0])

    except (OSError, FileNotFoundError) as oserr:  # if ffmpeg do not exist
        return ('Not found', oserr)

    return ('None', out[0])
# -----------------------------------------------------------#


def ff_conf(ffmpeg_url, ostype):
    """
    Receive output of the passed command to parse
    configuration messages of FFmpeg

    ...Returns lists of:
        - info = [version number etc, release and building]
        - others = [building flags]
        - enable = [enabled features]
        - disable = [disable features]

    ...If errors returns 'Not found'
    """

    # ------- grab generic informations:
    version = subp([ffmpeg_url, '-loglevel', 'error', '-version'], ostype)

    if 'Not found' in version[0]:
        return (version[0], version[1])

    enable, disable, others, conf, info = [], [], [], [], []

    for vers in version[1].split('\n'):
        if 'ffmpeg version' in vers:
            info.append(vers.strip())

        if 'built with' in vers:
            info.append(vers.strip())

    # ------- grab buildconf:
    build = subp([ffmpeg_url, '-loglevel', 'error', '-buildconf'], ostype)

    if 'Not found' in build[0]:
        return (build[0], build[1])

    for bld in build[1].split('\n'):
        conf.append(bld.strip())

    for enc in conf:
        if enc.startswith('--enable'):
            enable.append(enc.split('--enable-')[1])

        elif enc.startswith('--disable'):
            disable.append(enc.split('--disable-')[1])

        else:
            others.append(enc)

    if 'configuration:' in others:
        others.remove('configuration:')

    return (info, others, enable, disable)
# -------------------------------------------------------------------#


def ff_formats(ffmpeg_url, ostype):
    """
    Receive output of *ffmpeg -formats* command and return a
    ditionary with the follow keys and values:

        * KEYS                  * VALUES
        'Demuxing Supported' :  [list of (D)emuxing formats support]
        'Muxing Supported' :    [list of (M)uxing formats support]
        'Mux/Demux Supported' : [list of (D)emuxing(M)uxing formats support]
    """

    # ------- grab buildconf:
    ret = subp([ffmpeg_url, '-loglevel', 'error', '-formats'], ostype)

    if 'Not found' in ret[0]:
        return {ret[0]: ret[1]}

    frmt = ret[1].split('\n')

    dic = {'Demuxing Supported': [],
           'Muxing Supported': [],
           'Mux/Demux Supported': [],
           }
    for its in frmt:
        if its.strip().startswith('D '):
            dic['Demuxing Supported'].append(its.replace('D', '', 1).strip())
        elif its.strip().startswith('E '):
            dic['Muxing Supported'].append(its.replace('E', '', 1).strip())
        elif its.strip().startswith('DE '):
            dic['Mux/Demux Supported'].append(its.replace('DE', '', 1).strip())

    return dic
# -------------------------------------------------------------------#


def ff_codecs(ffmpeg_url, type_opt, ostype):
    """
    Receive output of *ffmpeg -encoders* or *ffmpeg -decoders*
    command and return a ditionary with the follow keys and values:

        * KEYS                  * VALUES
        'Video' :    [list of V.....   name    descrition]
        'Audio' :    [list of A.....   name    descrition]
        'Subtitle' : [list of S.....   name    descrition]

    Accept two arguments: the call to FFmpeg and the type of
    option used to obtain data on the encoding and decoding
    capabilities of FFmpeg. The options to pass as arguments
    are the following:

    -encoders
    -decoders
    """

    # ------- grab encoders or decoders output:
    ret = subp([ffmpeg_url, '-loglevel', 'error', type_opt], ostype)

    if 'Not found' in ret[0]:
        return ({ret[0], ret[1]})

    codecs = ret[1].split('\n')

    dic = {'Video': [], 'Audio': [], 'Subtitle': []}

    for sup in codecs:
        if sup.strip().startswith('V'):
            if 'V..... = Video' not in sup:
                dic['Video'].append(sup.strip())

        elif sup.strip().startswith('A'):
            if 'A..... = Audio' not in sup:
                dic['Audio'].append(sup.strip())

        elif sup.strip().startswith('S'):
            if 'S..... = Subtitle' not in sup:
                dic['Subtitle'].append(sup.strip())

    return dic
# -------------------------------------------------------------------#


def ff_topics(ffmpeg_url, topic, ostype):
    """
    Get output of the options help command of FFmpeg.
    Note that the 'topic' parameter is always a list.
    """

    # ------ get output:
    arr = [ffmpeg_url, '-loglevel', 'error'] + list(topic)
    ret = subp(arr, ostype)

    if 'Not found' in ret[0]:
        return (ret[0], ret[1])

    return ('None', ret[1])



# -*- coding: UTF-8 -*-
"""
Name: sequence_to_video.py
Porpose: A slideshow maker based on FFmpeg
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.08.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
import wx.lib.agw.hyperlink as hpl
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_dialogs.epilogue import Formula
from videomass.vdms_dialogs.filter_scale import Scale
from videomass.vdms_threads.ffprobe import ffprobe
from videomass.vdms_utils.utils import trailing_name_with_prog_digit
from videomass.vdms_utils.utils import time_to_integer
from videomass.vdms_utils.utils import integer_to_time


def check_images_size(flist):
    """
    Check for images size, if not equal return True,
    None otherwise.
    """
    sizes = []
    for index in flist:
        if 'video' in index.get('streams')[0]['codec_type']:
            width = index['streams'][0]['width']
            height = index['streams'][0]['height']
            sizes.append(f'{width}x{height}')

    if len(set(sizes)) > 1:
        wx.MessageBox(_('Images need to be resized, '
                        'please use Resize function.'),
                      'Videomass', wx.ICON_INFORMATION)
        return True

    return None
# -------------------------------------------------------------------------


class SequenceToVideo(wx.Panel):
    """
    This is an new implementation class to create
    Slideshow and static videos with the ability to
    add an audio track, resizing, and more.

    """
    VIOLET = '#D64E93'
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    MSG_1 = _("\n1. Import one or more image files such as JPG, PNG and BMP "
              "formats, then select one."
              "\n\n2. Use the Resize tool for images which "
              "have different sizes such as width and\nheight. "
              "It is optional in other cases."
              "\n\n3. Use the Timeline editor (CTRL+T) to set the time "
              "interval between images by adjusting\nthe \"End\" duration "
              "value and leaving the \"Start\" value at 00:00:00.000."
              "\n\n4. Run the conversion."
              "\n\n\nThe produced video will have the name of the selected "
              "file in the 'File List' panel, which\nwill be saved in a "
              "folder named 'Still_Images' with a progressive digit, "
              "in the path you specify.")
    # ---------------------------------------------------------------------

    def __init__(self, parent):
        """
        Simple GUI panel with few controls to create slideshows
        based on ffmpeg syntax.
        """
        self.parent = parent  # parent is the MainFrame
        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        icons = get.iconset
        self.ffprobe_cmd = self.appdata['ffprobe_cmd']
        self.opt = {"Scale": "", "Setdar": "", "Setsar": "",
                    "RESIZE": "", "ADuration": 0, "AudioMerging": "",
                    "Map": "-map 0:v?", "Shortest": ["", "Disabled"],
                    "Interval": "", "Clock": "00:00:00:000",
                    "Preinput": "1/0", "Fps": ["fps=10,", "10"],
                    }

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpresize = get_bmp(icons['scale'], ((16, 16)))
            bmpatrack = get_bmp(icons['atrack'], ((16, 16)))
            self.bmpreset = get_bmp(icons['clear'], ((16, 16)))
        else:
            bmpresize = wx.Bitmap(icons['scale'], wx.BITMAP_TYPE_ANY)
            bmpatrack = wx.Bitmap(icons['atrack'], wx.BITMAP_TYPE_ANY)
            self.bmpreset = wx.Bitmap(icons['clear'], wx.BITMAP_TYPE_ANY)

        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((20, 20))
        btn_help = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        btn_help.SetBackgroundColour(wx.Colour(SequenceToVideo.LGREEN))
        btn_help.SetForegroundColour(wx.Colour(SequenceToVideo.BLACK))
        sizer.Add(btn_help, 0, wx.ALL, 5)
        sizer.Add((20, 20))
        # sizer.Add((5, 5))
        boxctrl = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY), wx.VERTICAL)
        sizer.Add(boxctrl, 0, wx.ALL | wx.EXPAND, 5)
        siz_format = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_format)

        self.ckbx_static_img = wx.CheckBox(self, wx.ID_ANY,
                                           _('Enable a single still image'))
        boxctrl.Add(self.ckbx_static_img, 0, wx.ALL | wx.EXPAND, 5)
        siz_pict = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_pict)
        lbl_fps = wx.StaticText(self, wx.ID_ANY, label="FPS:")
        siz_pict.Add(lbl_fps, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.cmb_fps = wx.ComboBox(self, wx.ID_ANY,
                                   choices=[("None"),
                                            ("ntsc"),
                                            ("pal"),
                                            ("film"),
                                            ("23.976"),
                                            ("24"),
                                            ("25"),
                                            ("29.97"),
                                            ("30"),
                                            ("5"),
                                            ("10"),
                                            ("15"),
                                            ("60")
                                            ],
                                   size=(160, -1),
                                   style=wx.CB_DROPDOWN | wx.CB_READONLY
                                   )
        siz_pict.Add(self.cmb_fps, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.cmb_fps.SetSelection(10)
        self.btn_resize = wx.Button(self, wx.ID_ANY, _("Resize"),
                                    size=(-1, -1)
                                    )
        self.btn_resize.SetBitmap(bmpresize, wx.LEFT)
        siz_pict.Add(self.btn_resize, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.ckbx_far = wx.CheckBox(self, wx.ID_ANY,
                                    _('Force original aspect ratio using\n'
                                      'padding rather than stretching'))
        siz_pict.Add(self.ckbx_far, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.ckbx_far.Disable()
        siz_audio = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_audio)
        self.ckbx_audio = wx.CheckBox(self, wx.ID_ANY, _('Include audio file'))
        siz_audio.Add(self.ckbx_audio, 0, wx.ALL | wx.EXPAND, 5)
        self.ckbx_shortest = wx.CheckBox(self, wx.ID_ANY,
                                         _('Play the video until '
                                           'audio track finishes'
                                           ))
        siz_audio.Add(self.ckbx_shortest, 0, wx.ALL | wx.EXPAND, 5)
        self.ckbx_shortest.Disable()
        siz_afile = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_afile)
        self.btn_openaudio = wx.Button(self, wx.ID_ANY, _("Open audio file"),
                                       size=(-1, -1)
                                       )
        self.btn_openaudio.SetBitmap(bmpatrack, wx.LEFT)
        siz_afile.Add(self.btn_openaudio, 0, wx.ALL
                      | wx.ALIGN_CENTER_VERTICAL, 5)
        self.btn_openaudio.Disable()

        self.txt_apath = wx.TextCtrl(self, wx.ID_ANY, size=(500, -1),
                                     style=wx.TE_READONLY
                                     )
        siz_afile.Add(self.txt_apath, 0, wx.ALL, 5)
        self.txt_apath.Disable()

        siz_addparams = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_addparams, 0, wx.EXPAND, 0)

        self.ckbx_edit = wx.CheckBox(self, wx.ID_ANY,
                                     _('Additional arguments'))
        siz_addparams.Add(self.ckbx_edit, 0, wx.ALL, 5)
        self.txt_addparams = wx.TextCtrl(self, wx.ID_ANY,
                                         value=('-c:v libx264 -crf 23 '
                                                '-tune:v stillimage'),
                                         size=(-1, -1),)
        siz_addparams.Add(self.txt_addparams, 1, wx.ALL | wx.EXPAND, 5)
        self.txt_addparams.Disable()
        sizer.Add((20, 20))
        fgs1 = wx.BoxSizer(wx.VERTICAL)
        sizer_link1 = wx.BoxSizer(wx.HORIZONTAL)
        fgs1.Add(sizer_link1)
        lbl_link = wx.StaticText(self, wx.ID_ANY,
                                 label=_("For more information, "
                                         "visit the official FFmpeg "
                                         "documentation:")
                                 )
        link1 = hpl.HyperLinkCtrl(self, -1, ("FFmpeg Slideshow wiki"),
                                  URL="https://trac.ffmpeg.org/wiki/Slideshow"
                                  )
        sizer_link1.Add(lbl_link, 0, wx.ALL | wx.EXPAND, 5)
        sizer_link1.Add(link1)
        sizer.Add(fgs1, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer)

        if self.appdata['ostype'] == 'Darwin':
            lbl_link.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            lbl_link.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_CHECKBOX, self.on_enable_audio, self.ckbx_audio)
        self.Bind(wx.EVT_BUTTON, self.on_addaudio_track, self.btn_openaudio)
        self.Bind(wx.EVT_CHECKBOX, self.on_shortest, self.ckbx_shortest)
        self.Bind(wx.EVT_CHECKBOX, self.on_addparams, self.ckbx_edit)
        self.Bind(wx.EVT_CHECKBOX, self.on_force_aspect_ratio, self.ckbx_far)
        self.Bind(wx.EVT_COMBOBOX, self.on_fps, self.cmb_fps)
        self.Bind(wx.EVT_BUTTON, self.on_resizing, self.btn_resize)
    # ---------------------------------------------------------

    def on_help(self, event):
        """
        event on button help
        """
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   SequenceToVideo.MSG_1,
                                   SequenceToVideo.LGREEN,
                                   SequenceToVideo.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # ------------------------------------------------------------------#

    def reset_all_values(self):
        """
        Reset all control and dictionary values
        NOTE this method is not used for now
        """
        self.opt["Scale"], self.opt["Setdar"] = "", ""
        self.opt["Setsar"], self.opt["RESIZE"] = "", ""
        self.opt["Preinput"] = "1/0"
        self.opt["Shortest"] = ["", "Disabled"]
        self.opt["Interval"] = ""
        self.opt["Clock"] = "00:00:00.000"
        self.ckbx_static_img.SetValue(False)
        self.btn_resize.SetBackgroundColour(wx.NullColour)
        self.ckbx_audio.SetValue(False)
        self.on_enable_audio(self)
        self.ckbx_shortest.SetValue(False)
    # ---------------------------------------------------------

    def on_addparams(self, event):
        """
        Enable additional args to command line
        """
        if self.ckbx_edit.IsChecked():
            self.txt_addparams.Enable()
        else:
            self.txt_addparams.Disable()
    # ---------------------------------------------------------

    def on_force_aspect_ratio(self, event):
        """
        Adds extra params to scale filter to preserving
        images aspect ratio.

        """
        if self.ckbx_far.IsChecked():
            width = self.opt["Scale"].split(':', maxsplit=1)[0][8:]
            height = self.opt["Scale"].split(':', maxsplit=1)[1][2:]
            params = (f':force_original_aspect_ratio=decrease:eval=frame,'
                      f'pad={width}:{height}:-1:-1:eval=frame')
            scale = ''.join(self.opt["Scale"] + params)
            if self.opt["Setdar"]:
                scale = f'{scale},{self.opt["Setdar"]}'
            if self.opt["Setsar"]:
                scale = f'{scale},{self.opt["Setsar"]}'
            self.opt["RESIZE"] = f'-vf "{scale}"'
        else:
            scale = self.opt["Scale"]
            if self.opt["Setdar"]:
                scale = f'{scale},{self.opt["Setdar"]}'
            if self.opt["Setsar"]:
                scale = f'{scale},{self.opt["Setsar"]}'
            self.opt["RESIZE"] = f'-vf "{scale}"'
    # ------------------------------------------------------------------#

    def file_selection(self):
        """
        Gets the selected file on files list and returns an object
        of type list [str('selected file name'), int(index)].
        Returns None if no files are selected.

        """
        if len(self.parent.file_src) == 1:
            return (self.parent.file_src[0], 0)

        if not self.parent.filedropselected:
            wx.MessageBox(_("Have to select an item in the file list first"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return None

        clicked = self.parent.filedropselected
        return (clicked, self.parent.file_src.index(clicked))
    # ------------------------------------------------------------------#

    def get_video_stream(self):
        """
        Given a frame or a video file, it returns a tuple of data
        containing information on the streams required by some video
        filters.

        """
        fget = self.file_selection()
        if not fget:
            return None

        index = self.parent.data_files[fget[1]]

        if 'video' in index.get('streams')[0]['codec_type']:
            width = int(index['streams'][0]['width'])
            height = int(index['streams'][0]['height'])
            filename = index['format']['filename']
            duration = index['format'].get('time', '00:00:00.000')
            return dict(zip(['width', 'height', 'filename', 'duration'],
                            [width, height, filename, duration]))

        wx.MessageBox(_('The file is not a frame or a video file'),
                      _('Videomass - Warning!'), wx.ICON_WARNING)
        return None
    # ------------------------------------------------------------------#

    def on_resizing(self, event):
        """
        Enable or disable scale, setdar and setsar filters
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with Scale(self,
                   self.opt["Scale"],
                   self.opt["Setdar"],
                   self.opt["Setsar"],
                   self.bmpreset,
                   **kwa,
                   ) as sizing:

            if sizing.ShowModal() == wx.ID_OK:
                data = sizing.getvalue()
                if not [x for x in data.values() if x]:
                    self.btn_resize.SetBackgroundColour(wx.NullColour)
                    self.opt["Setdar"] = ""
                    self.opt["Setsar"] = ""
                    self.opt["Scale"] = ""
                    self.opt["RESIZE"] = ''
                    self.ckbx_far.SetValue(False)
                    self.ckbx_far.Disable()
                else:
                    self.btn_resize.SetBackgroundColour(
                        wx.Colour(SequenceToVideo.VIOLET))
                    self.opt["Scale"] = data['scale']
                    self.opt['Setdar'] = data['setdar']
                    self.opt['Setsar'] = data['setsar']

                    flt = ''.join([f'{x},' for x in data.values() if x])[:-1]
                    if flt:
                        self.opt["RESIZE"] = f'-vf "{flt}"'
                        if '=-1' in data['scale'] or '=-2' in data['scale']:
                            self.ckbx_far.SetValue(False)
                            self.ckbx_far.Disable()
                        else:
                            self.ckbx_far.Enable()
                        if self.ckbx_far.IsChecked():
                            self.on_force_aspect_ratio(self)
    # ---------------------------------------------------------

    def on_fps(self, event):
        """
        Set frame per seconds using ComboBox
        """
        val = self.cmb_fps.GetValue()
        if val == 'None':
            self.opt["Fps"] = ['', 'None']
        else:
            self.opt["Fps"] = [f'fps={val},', val]
    # ---------------------------------------------------------

    def on_shortest(self, event):
        """
        Enable or disable shortest when audio file is loaded.
        Always disabled otherwise.
        """
        if self.ckbx_shortest.IsChecked():
            self.opt["Shortest"] = ["", "Disabled"]
        else:
            self.opt["Shortest"] = ["-shortest", "Enabled"]
    # ---------------------------------------------------------

    def on_enable_audio(self, event):
        """
        Enables controls to create a video file from a
        sequence of images using Concat Demuxer.
        """
        if self.ckbx_audio.IsChecked():
            self.btn_openaudio.Enable()
            self.txt_apath.Enable()
            self.ckbx_shortest.Enable()
            self.opt["Shortest"] = ["-shortest", "Enabled"]
        else:
            self.btn_openaudio.Disable()
            self.txt_apath.Disable()
            self.txt_apath.Clear()
            self.ckbx_shortest.Disable()
            self.ckbx_shortest.SetValue(False)
            self.btn_openaudio.SetBackgroundColour(wx.NullColour)
            self.btn_openaudio.SetLabel(_("Open audio file"))
            self.opt["AudioMerging"] = ''
            self.opt["Map"] = '-map 0:v?'
            self.opt["ADuration"] = 0
            self.opt["Shortest"] = ["", "Disabled"]
    # ---------------------------------------------------------

    def on_addaudio_track(self, event):
        """
        Add audio track to video.
        """
        fmt = ('*.wav;*.aiff;*.flac;*.oga;*.ogg;*.opus;*.tta;*.m4a;'
               '*.aac;*.ac3;*.mp3;')
        wild = f"Audio source ({fmt})|{fmt}| All files (*.*)|*.*"

        with wx.FileDialog(self, _("Open Audio File"),
                           wildcard=wild,
                           style=wx.FD_OPEN
                           | wx.FD_FILE_MUST_EXIST) as fdlg:

            if fdlg.ShowModal() == wx.ID_CANCEL:
                return
            pathname = fdlg.GetPath()

        probe = ffprobe(pathname, self.ffprobe_cmd, hide_banner=None)

        if probe[1]:  # some error
            msg = _("Invalid file: '{}'\n\n{}").format(pathname, probe[1])
            wx.MessageBox(msg, _('Videomass - Error!'), wx.ICON_ERROR, self)
            return

        if probe[0]['streams'][0]['codec_type'] != 'audio':
            msg = _("Invalid file: '{}'\n\n"
                    "It doesn't appear to be an audio file.").format(pathname)
            wx.MessageBox(msg, _('Videomass - Error!'), wx.ICON_ERROR, self)
            return

        self.btn_openaudio.SetBackgroundColour(
            wx.Colour(SequenceToVideo.VIOLET))
        ext = os.path.splitext(pathname)[1].replace('.', '').upper()
        self.btn_openaudio.SetLabel(ext)
        self.txt_apath.write(pathname)
        self.opt["AudioMerging"] = f'-i "{pathname}"'
        self.opt["Map"] = '-map 0:v:0 -map 1:a:0'
        mills = float(probe[0]['format']['duration']) * 1000
        self.opt["ADuration"] = round(mills)
    # ---------------------------------------------------------

    def build_command_slideshow(self, timeline):
        """
        Set ffmpeg arguments for a slideshow.
        """
        loop = ''
        if self.ckbx_audio.IsChecked():
            if self.opt["Shortest"][0]:
                sec = time_to_integer(timeline, sec=True, rnd=True)
                duration = sec * len(self.parent.file_src)
                self.opt["Clock"] = integer_to_time(duration * 1000)
            else:
                self.opt["Clock"] = integer_to_time(self.opt["ADuration"])
                sec = time_to_integer(timeline, sec=True, rnd=True)
                duration = self.opt["ADuration"]
                loop = f'-loop 1 -t {self.opt["Clock"]}'
        else:
            sec = time_to_integer(timeline, sec=True, rnd=True)
            duration = sec * len(self.parent.file_src)
            self.opt["Clock"] = integer_to_time(duration * 1000)

        framerate = '-framerate 1/1' if not sec else f'-framerate 1/{sec}'
        self.opt["Preinput"] = f'{loop} {framerate}'
        self.opt["Interval"] = sec

        if self.txt_addparams.IsEnabled():
            addparam = self.txt_addparams.GetValue()
        else:
            addparam = ''

        cmd_2 = (f'{self.opt["AudioMerging"]} {addparam} -vf '
                 f'"{self.opt["Fps"][0]}format=yuv420p" {self.opt["Map"]} '
                 f'{self.opt["Shortest"][0]}')

        return cmd_2, duration
    # ---------------------------------------------------------

    def build_command_loop_image(self, timeline):
        """
        Set ffmpeg arguments for a static video,
        the checkbox `create a static video from image` is checked.

        """
        if self.ckbx_audio.IsChecked():
            if self.opt["Shortest"][0]:
                sec = time_to_integer(timeline, sec=True, rnd=True)
                duration = sec
                self.opt["Clock"] = integer_to_time(duration * 1000)
            else:
                self.opt["Clock"] = integer_to_time(self.opt["ADuration"])
                duration = self.opt["ADuration"]
                sec = round(self.opt["ADuration"] / 1000)
        else:
            sec = time_to_integer(timeline, sec=True, rnd=True)
            duration = sec
            self.opt["Clock"] = integer_to_time(duration * 1000)

        self.opt["Preinput"] = f'-loop 1 -t {self.opt["Clock"]}'
        self.opt["Interval"] = sec

        if self.txt_addparams.IsEnabled():
            addparam = self.txt_addparams.GetValue()
        else:
            addparam = ''

        cmd_2 = (f'{self.opt["AudioMerging"]} {addparam} -vf '
                 f'"{self.opt["Fps"][0]}format=yuv420p" {self.opt["Map"]} '
                 f'{self.opt["Shortest"][0]}')

        return cmd_2, duration
    # ---------------------------------------------------------

    def check_to_slide(self, fsource):
        """
        Check compatibility between loaded images and files exist.
        """
        itemcount = self.parent.fileDnDTarget.flCtrl.GetItemCount()
        for itc in range(itemcount):
            typemedia = self.parent.fileDnDTarget.flCtrl.GetItemText(itc, 3)
            if 'video' not in typemedia or 'sequence' not in typemedia:
                wx.MessageBox(_("Invalid file: '{}'").format(fsource[itc]),
                              _('Videomass - Error!'), wx.ICON_ERROR, self)
                return True

        unsupp = [f for f in fsource if os.path.splitext(f)[1]
                  not in ('.jpeg', '.jpg', '.png', '.bmp',
                          '.JPEG', '.JPG', '.PNG', '.BMP')
                  ]
        if unsupp:
            ext = os.path.splitext(unsupp[0])[1]
            wx.MessageBox(_("Unsupported format '{}'").format(ext),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return True

        for fsrc in fsource:
            if not os.path.isfile(os.path.abspath(fsrc)):
                wx.MessageBox(_('File does not exist:\n\n"{}"\n').format(fsrc),
                              _('ERROR'), wx.ICON_ERROR, self)
                return True
        return None
    # ---------------------------------------------------------

    def check_to_loop(self, typemedia, clicked):
        """
        Check the compatibility of the selected image and file exist.
        """
        if 'video' not in typemedia or 'sequence' not in typemedia:
            wx.MessageBox(_("Invalid file: '{}'").format(clicked),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return True

        supp = ('.jpeg', '.jpg', '.png', '.bmp',
                '.JPEG', '.JPG', '.PNG', '.BMP')
        ext = os.path.splitext(clicked)[1]
        if ext not in supp:
            wx.MessageBox(_("Unsupported format '{}'").format(ext),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return True

        if not os.path.isfile(os.path.abspath(clicked)):
            wx.MessageBox(_('File does not exist:'
                            '\n\n"{}"\n').format(clicked),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return True
        return None
    # ---------------------------------------------------------

    def get_args_line(self):
        """
        get arguments line for 'loop' or 'slide' modes
        """
        if not self.parent.time_seq:
            timeline = '00:00:01.000'
        else:
            timeline = self.parent.time_seq.split()[3]

        if self.ckbx_static_img.IsChecked():
            args = self.build_command_loop_image(timeline)
        else:
            args = self.build_command_slideshow(timeline)

        return args
    # ---------------------------------------------------------

    def on_start(self):
        """
        Redirect to `switch_to_processing`
        """
        fget = self.file_selection()
        if not fget:
            return

        fsource = self.parent.file_src

        if self.appdata['outputdir_asinput']:
            destdir = os.path.dirname(fget[0])
        else:
            destdir = self.appdata['outputdir']

        name = self.parent.fileDnDTarget.flCtrl.GetItemText(fget[1], 5)
        outputdir = trailing_name_with_prog_digit(destdir, 'Still_Images')

        if outputdir[0] == 'ERROR':
            wx.MessageBox(f"{outputdir[1]}", _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        destdir = os.path.join(outputdir, f"{name}.mkv")

        if self.ckbx_static_img.IsChecked():
            countmax = 1
            files = (fget[0],)
            prop = self.parent.fileDnDTarget.flCtrl.GetItemText(fget[1], 3)
            if self.check_to_loop(prop, fget[0]):
                return
        else:
            files = fsource
            countmax = len(files)
            if self.check_to_slide(files):
                return
            if not self.opt["RESIZE"] and countmax != 1:
                if check_images_size(self.parent.data_files):
                    return

        args = self.get_args_line()  # get args for command line
        kwargs = {'logname': 'Still Image Maker.log',
                  'type': 'sequence_to_video', 'source': files,
                  'destination': destdir, 'outputdir': outputdir,
                  'args': args[0], 'nmax': countmax, 'duration': args[1],
                  'pre-input-1': self.opt["Preinput"],
                  'resize': self.opt["RESIZE"],
                  'start-time': '', 'end-time': '',
                  'preset name': 'Still Image Maker',
                  }

        keyval = self.update_dict(f"{name}.mkv", outputdir, countmax, 'mkv')
        ending = Formula(self, (700, 320),
                         self.parent.movetotrash,
                         self.parent.emptylist,
                         **keyval,
                         )
        if ending.ShowModal() == wx.ID_OK:
            (self.parent.movetotrash,
             self.parent.emptylist) = ending.getvalue()
        else:
            return

        try:
            os.makedirs(outputdir, mode=0o777)
        except (OSError, FileExistsError) as err:
            wx.MessageBox(f"{err}", _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        self.parent.switch_to_processing(kwargs["type"],
                                         kwargs["logname"],
                                         datalist=kwargs
                                         )
        return
    # -----------------------------------------------------------

    def update_dict(self, newfile, destdir, count, ext):
        """
        Update information before send to epilogue

        """
        time = self.opt["Clock"]
        resize = _('Enabled') if self.opt["RESIZE"] else _('Disabled')
        short = _(self.opt["Shortest"][1])
        preinput = self.opt["Preinput"]
        duration = self.opt["Interval"]
        if self.ckbx_edit.IsChecked():
            addargs = self.txt_addparams.GetValue()
        else:
            addargs = ''

        keys = (_("Items to concatenate\nOutput filename"
                  "\nDestination\nOutput Format"
                  "\nAdditional arguments"
                  "\nAudio file\nShortest\nResize\nPre-input"
                  "\nFrame per Second (FPS)\nStill image duration"
                  "\nOverall video duration"
                  ))
        vals = (f'{count}\n{newfile}\n{destdir}\n{ext}\n{addargs}'
                f'\n{self.txt_apath.GetValue()}\n{short}'
                f'\n{resize}\n{preinput}\n{self.opt["Fps"][1]}'
                f'\n{duration} Seconds\n{time}'
                )
        return {'key': keys, 'val': vals}



# -*- coding: UTF-8 -*-

"""
Name: presets_manager.py
Porpose: ffmpeg's presets manager panel
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.22.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import time
import os
import sys
import wx
import wx.lib.scrolledpanel as scrolled
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_utils.presets_manager_utils import json_data
from videomass.vdms_utils.presets_manager_utils import supported_formats
from videomass.vdms_utils.presets_manager_utils import delete_profiles
from videomass.vdms_utils.presets_manager_utils import update_oudated_profiles
from videomass.vdms_utils.presets_manager_utils import write_new_profile
from videomass.vdms_utils.utils import copy_restore
from videomass.vdms_utils.utils import copy_on
from videomass.vdms_utils.utils import copydir_recursively
from videomass.vdms_utils.utils import copy_missing_data
from videomass.vdms_utils.utils import update_timeseq_duration
from videomass.vdms_io.checkup import check_files
from videomass.vdms_dialogs import setting_profiles
from videomass.vdms_dialogs.epilogue import Formula


class PrstPan(wx.Panel):
    """
    Interface for using and managing presets in the FFmpeg syntax.

    """
    # set colour in RGB rappresentetion:
    AZURE_NEON = 158, 201, 232
    # set colour in HTML rappresentetion:
    AZURE = '#15a6a6'  # or rgb form (wx.Colour(217,255,255))
    YELLOW = '#bd9f00'
    RED = '#ea312d'
    ORANGE = '#f28924'
    GREENOLIVE = '#8aab3c'
    GREEN = '#268826'
    LIMEGREEN = '#87A615'
    TROPGREEN = '#15A660'
    WHITE = '#fbf4f4'  # white for background status bar
    BLACK = '#060505'  # black for background status bar
    # -----------------------------------------------------------------

    def __init__(self, parent):
        """
        Each presets is a JSON file (Javascript object notation) which is
        a list object with a variable number of items (called profiles)
        of type <class 'dict'>, each of which collect 5 keys object in
        the following form:

        {'Name': "",
        "Descritpion": "",
        "First_pass": "",
        "Second_pass": "",
        "Supported_list": "",
        "Output_extension": "",
        }
        """
        self.parent = parent  # parent is the MainFrame
        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        icons = get.iconset
        self.src_prst = os.path.join(self.appdata['srcpath'], 'presets')
        self.user_prst = os.path.join(self.appdata['confdir'], 'presets')
        self.array = []  # Parameters of the selected profile
        self.txtcmdedited = True  # show warning if cmdline is edited
        self.check_presets_version = False  # see `update_preset_state`

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpnewprf = get_bmp(icons['profile_add'], ((16, 16)))
            bmpeditprf = get_bmp(icons['profile_edit'], ((16, 16)))
            bmpdelprf = get_bmp(icons['profile_del'], ((16, 16)))
            bmpcopyprf = get_bmp(icons['profile_copy'], ((16, 16)))
        else:
            bmpnewprf = wx.Bitmap(icons['profile_add'], wx.BITMAP_TYPE_ANY)
            bmpeditprf = wx.Bitmap(icons['profile_edit'], wx.BITMAP_TYPE_ANY)
            bmpdelprf = wx.Bitmap(icons['profile_del'], wx.BITMAP_TYPE_ANY)
            bmpcopyprf = wx.Bitmap(icons['profile_copy'], wx.BITMAP_TYPE_ANY)

        prst = sorted([os.path.splitext(x)[0] for x in
                       os.listdir(self.user_prst) if
                       os.path.splitext(x)[1] == '.json'
                       ])
        wx.Panel.__init__(self, parent, -1)

        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizer_div = wx.BoxSizer(wx.HORIZONTAL)
        sizer_base.Add(sizer_div, 1, wx.EXPAND)
        # ------- BOX PRESETS
        boxpresets = wx.StaticBoxSizer(wx.StaticBox(
            self, wx.ID_ANY, _('Presets')), wx.VERTICAL)
        sizer_div.Add(boxpresets, 0, wx.ALL | wx.EXPAND, 5)
        self.cmbx_prst = wx.ComboBox(self, wx.ID_ANY,
                                     choices=prst,
                                     size=(200, -1),
                                     style=wx.CB_DROPDOWN
                                     | wx.CB_READONLY,
                                     )
        boxpresets.Add(self.cmbx_prst, 0, wx.ALL | wx.EXPAND, 5)
        boxpresets.Add((5, 5))
        line0 = wx.StaticLine(self, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        boxpresets.Add(line0, 0, wx.ALL | wx.EXPAND, 5)
        boxpresets.Add((5, 5))
        panelscr = scrolled.ScrolledPanel(self, -1, size=(230, 500),
                                          style=wx.TAB_TRAVERSAL
                                          | wx.BORDER_THEME,
                                          name="panelscroll",
                                          )
        fgs1 = wx.BoxSizer(wx.VERTICAL)
        self.btn_newpreset = wx.Button(panelscr, wx.ID_ANY,
                                       _("New"), size=(-1, -1))
        fgs1.Add(self.btn_newpreset, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_delpreset = wx.Button(panelscr, wx.ID_ANY,
                                       _("Remove"), size=(-1, -1))
        fgs1.Add(self.btn_delpreset, 0, wx.ALL | wx.EXPAND, 5)
        line1 = wx.StaticLine(panelscr, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        fgs1.Add((5, 5))
        fgs1.Add(line1, 0, wx.ALL | wx.EXPAND, 5)
        fgs1.Add((5, 5))
        self.btn_savecopy = wx.Button(panelscr, wx.ID_ANY,
                                      _("Export selected"), size=(-1, -1))
        fgs1.Add(self.btn_savecopy, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_saveall = wx.Button(panelscr, wx.ID_ANY,
                                     _("Export all..."), size=(-1, -1))
        fgs1.Add(self.btn_saveall, 0, wx.ALL | wx.EXPAND, 5)

        line2 = wx.StaticLine(panelscr, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        fgs1.Add((5, 5))
        fgs1.Add(line2, 0, wx.ALL | wx.EXPAND, 5)
        fgs1.Add((5, 5))
        self.btn_restore = wx.Button(panelscr, wx.ID_ANY,
                                     _("Import preset"), size=(-1, -1))
        fgs1.Add(self.btn_restore, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_restoreall = wx.Button(panelscr, wx.ID_ANY,
                                        _("Import folder"), size=(-1, -1))
        fgs1.Add(self.btn_restoreall, 0, wx.ALL | wx.EXPAND, 5)

        line3 = wx.StaticLine(panelscr, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        fgs1.Add((5, 5))
        fgs1.Add(line3, 0, wx.ALL | wx.EXPAND, 5)
        fgs1.Add((5, 5))
        self.btn_restoredef = wx.Button(panelscr, wx.ID_ANY,
                                        _("Restore selected"), size=(-1, -1))
        fgs1.Add(self.btn_restoredef, 0, wx.ALL | wx.EXPAND, 5)

        self.btn_restorealldefault = wx.Button(panelscr, wx.ID_ANY,
                                               _("Restore all..."),
                                               size=(-1, -1)
                                               )
        fgs1.Add(self.btn_restorealldefault, 0, wx.ALL | wx.EXPAND, 5)
        line4 = wx.StaticLine(panelscr, wx.ID_ANY, pos=wx.DefaultPosition,
                              size=wx.DefaultSize, style=wx.LI_HORIZONTAL,
                              name=wx.StaticLineNameStr
                              )
        fgs1.Add((5, 5))
        fgs1.Add(line4, 0, wx.ALL | wx.EXPAND, 5)
        fgs1.Add((5, 5))
        self.btn_refresh = wx.Button(panelscr, wx.ID_ANY,
                                     _("Reload all"), size=(-1, -1))
        fgs1.Add(self.btn_refresh, 0, wx.ALL | wx.EXPAND, 5)
        boxpresets.Add(panelscr, 0, wx.ALL | wx.CENTRE, 5)
        panelscr.SetSizer(fgs1)
        panelscr.SetAutoLayout(1)
        panelscr.SetupScrolling()
        # ------ LIST CONTROL & BOX PROFILES
        # --- listctrl
        self.lctrl = wx.ListCtrl(self, wx.ID_ANY,
                                 style=wx.LC_REPORT
                                 | wx.SUNKEN_BORDER
                                 | wx.LC_SINGLE_SEL,
                                 )
        boxprofiles = wx.StaticBoxSizer(wx.StaticBox(
            self, wx.ID_ANY, _('Profiles')), wx.VERTICAL)
        boxprofiles.Add(self.lctrl, 1, wx.ALL | wx.EXPAND, 5)
        # --- profile buttons
        grid_profiles = wx.FlexGridSizer(0, 4, 0, 5)
        self.btn_newprofile = wx.Button(self, wx.ID_ANY,
                                        _("Add"), size=(-1, -1))
        self.btn_newprofile.SetBitmap(bmpnewprf, wx.LEFT)
        grid_profiles.Add(self.btn_newprofile, 0, wx.ALL, 0)
        self.btn_delprofile = wx.Button(self, wx.ID_ANY,
                                        _("Delete"), size=(-1, -1))
        self.btn_delprofile.SetBitmap(bmpdelprf, wx.LEFT)
        self.btn_delprofile.Disable()
        grid_profiles.Add(self.btn_delprofile, 0, wx.ALL, 0)
        self.btn_editprofile = wx.Button(self, wx.ID_ANY,
                                         _("Edit"), size=(-1, -1))
        self.btn_editprofile.SetBitmap(bmpeditprf, wx.LEFT)
        self.btn_editprofile.Disable()
        grid_profiles.Add(self.btn_editprofile, 0, wx.ALL, 0)
        self.btn_copyprofile = wx.Button(self, wx.ID_ANY,
                                         _("Duplicate"), size=(-1, -1))
        self.btn_copyprofile.SetBitmap(bmpcopyprf, wx.LEFT)
        self.btn_copyprofile.Disable()
        grid_profiles.Add(self.btn_copyprofile, 0, wx.ALL, 0)
        boxprofiles.Add(grid_profiles, 0, wx.ALL, 5)
        sizer_div.Add(boxprofiles, 1, wx.ALL | wx.EXPAND, 5)
        # ------- command line
        grd_cmd = wx.BoxSizer(wx.HORIZONTAL)
        sizer_base.Add(grd_cmd, 0, wx.EXPAND)
        sbox1 = wx.StaticBox(self, wx.ID_ANY, _("One-Pass Encoding"))
        box_cmd1 = wx.StaticBoxSizer(sbox1, wx.VERTICAL)
        grd_cmd.Add(box_cmd1, 1, wx.ALL | wx.EXPAND, 5)
        self.txt_1cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                    size=(-1, 120), style=wx.TE_MULTILINE
                                    | wx.TE_PROCESS_ENTER,
                                    )
        box_cmd1.Add(self.txt_1cmd, 1, wx.ALL | wx.EXPAND, 5)
        self.pass_1_pre = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_PROCESS_ENTER,
                                      )
        box_cmd1.Add(self.pass_1_pre, 0, wx.ALL | wx.EXPAND, 5)
        sbox2 = wx.StaticBox(self, wx.ID_ANY, _("Two-Pass Encoding"))
        box_cmd2 = wx.StaticBoxSizer(sbox2, wx.VERTICAL)
        grd_cmd.Add(box_cmd2, 1, wx.ALL | wx.EXPAND, 5)
        self.txt_2cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                    size=(-1, 120), style=wx.TE_MULTILINE
                                    | wx.TE_PROCESS_ENTER,
                                    )
        box_cmd2.Add(self.txt_2cmd, 1, wx.ALL | wx.EXPAND, 5)
        self.pass_2_pre = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_PROCESS_ENTER,
                                      )
        box_cmd2.Add(self.pass_2_pre, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer_base)
        self.Layout()

        if self.appdata['ostype'] == 'Darwin':
            self.pass_1_pre.SetFont(wx.Font(10, wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL, wx.NORMAL))
            self.pass_2_pre.SetFont(wx.Font(10, wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL, wx.NORMAL))
            self.txt_1cmd.SetFont(wx.Font(10, wx.FONTFAMILY_TELETYPE,
                                          wx.NORMAL, wx.NORMAL))
            self.txt_2cmd.SetFont(wx.Font(10, wx.FONTFAMILY_TELETYPE,
                                          wx.NORMAL, wx.NORMAL))
        else:
            self.pass_1_pre.SetFont(wx.Font(8, wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL, wx.NORMAL))
            self.pass_2_pre.SetFont(wx.Font(8, wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL, wx.NORMAL))
            self.txt_1cmd.SetFont(wx.Font(8, wx.FONTFAMILY_TELETYPE,
                                          wx.NORMAL, wx.NORMAL))
            self.txt_2cmd.SetFont(wx.Font(8, wx.FONTFAMILY_TELETYPE,
                                          wx.NORMAL, wx.NORMAL))

        # ------- tipips
        self.cmbx_prst.SetToolTip(_("Choose a preset and view its profiles"))
        tip = _("Create a new profile and save it in the selected preset")
        self.btn_newprofile.SetToolTip(tip)
        self.btn_delprofile.SetToolTip(_("Delete the selected profile"))
        self.btn_editprofile.SetToolTip(_("Edit the selected profile"))
        tip = _("Create a new preset")
        self.btn_newpreset.SetToolTip(tip)
        tip = _("Remove the selected preset from the Presets Manager")
        self.btn_delpreset.SetToolTip(tip)
        tip = _("Export selected preset as copy")
        self.btn_savecopy.SetToolTip(tip)
        tip = _("Export entire presets folder as copy")
        self.btn_saveall.SetToolTip(tip)
        tip = _("Import a new preset or update an existing one")
        self.btn_restore.SetToolTip(tip)
        tip = _("Import a presets folder, updating those in use")
        self.btn_restoreall.SetToolTip(tip)
        tip = _("Replace the selected preset with the Videomass default one")
        self.btn_restoredef.SetToolTip(tip)
        tip = _("Retrieve all Videomass default presets")
        self.btn_restorealldefault.SetToolTip(tip)
        self.btn_refresh.SetToolTip(_("Update the presets list"))
        tip = _('FFmpeg arguments for one-pass encoding')
        self.txt_1cmd.SetToolTip(tip)
        tip = _('FFmpeg arguments for two-pass encoding')
        self.txt_2cmd.SetToolTip(tip)
        tip = (_('Any optional arguments to add before input file on the '
                 'one-pass encoding, e.g required names of some hardware '
                 'accelerations like -hwaccel to use with CUDA.'))
        self.pass_1_pre.SetToolTip(tip)
        tip = (_('Any optional arguments to add before input file on the '
                 'two-pass encoding, e.g required names of some hardware '
                 'accelerations like -hwaccel to use with CUDA.'))
        self.pass_2_pre.SetToolTip(tip)

        # ----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_COMBOBOX, self.on_preset_selection, self.cmbx_prst)
        self.Bind(wx.EVT_BUTTON, self.profile_add, self.btn_newprofile)
        self.Bind(wx.EVT_BUTTON, self.profile_del, self.btn_delprofile)
        self.Bind(wx.EVT_BUTTON, self.profile_edit, self.btn_editprofile)
        self.Bind(wx.EVT_BUTTON, self.profile_copy, self.btn_copyprofile)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.lctrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.lctrl)
        self.Bind(wx.EVT_BUTTON, self.preset_new, self.btn_newpreset)
        self.Bind(wx.EVT_BUTTON, self.preset_del, self.btn_delpreset)
        self.Bind(wx.EVT_BUTTON, self.preset_export, self.btn_savecopy)
        self.Bind(wx.EVT_BUTTON, self.preset_export_all, self.btn_saveall)
        self.Bind(wx.EVT_BUTTON, self.preset_import, self.btn_restore)
        self.Bind(wx.EVT_BUTTON, self.preset_default, self.btn_restoredef)
        self.Bind(wx.EVT_BUTTON, self.preset_import_all, self.btn_restoreall)
        self.Bind(wx.EVT_BUTTON, self.preset_default_all,
                  self.btn_restorealldefault)
        self.Bind(wx.EVT_BUTTON, self.presets_refresh, self.btn_refresh)

        # ---------------------------- defaults
        self.cmbx_prst.SetSelection(0),
        self.set_listctrl(self.appdata['prstmng_column_width'])
    # ----------------------------------------------------------------------

    def update_preset_state(self):
        """
        Check the version of the installed presets (inside conf dir).
        If the preset database is updatable, it asks the user for
        his confirmation.
        """
        if self.check_presets_version:
            return

        srctext = os.path.join(self.src_prst, 'version', 'version.txt')
        conftext = os.path.join(self.user_prst, 'version', 'version.txt')
        if not os.path.isfile(conftext) or not os.path.isfile(srctext):
            return

        with open(conftext, "r", encoding='utf-8') as vers:
            confversion = vers.read().strip()
        with open(srctext, "r", encoding='utf-8') as vers:
            srcversion = vers.read().strip()

        old = sum((int(x) for x in confversion.split('.')))
        updated = sum((int(x) for x in srcversion.split('.')))
        self.check_presets_version = True

        if updated > old:
            msg = _('Outdated presets version found: v{1}\n'
                    'A new version is available: v{0}\n\n'
                    'This update provides new presets included on the '
                    'latest versions of Videomass.\n\n'
                    'To avoid data loss and allow for possible recovery, '
                    'the outdated presets folder will be backed up in the '
                    'program configuration directory: "{2}"\n\n'
                    'Do you want to perform this '
                    'update now?').format(srcversion,
                                          confversion,
                                          self.appdata["confdir"])
            if wx.MessageBox(msg, _('Please confirm'), wx.ICON_QUESTION
                             | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                return
            err = self.preset_import_all(event=None)
            if err:
                return

            # update version.txt file to latest version
            with open(conftext, "w", encoding='utf-8') as updatevers:
                updatevers.write(f'{srcversion}\n')

            # copies missing file/dir to the destination directory
            copy_missing_data(self.src_prst, self.user_prst)
    # --------------------------------------------------------------------

    def reset_list(self, reset_cmbx=False):
        """
        Clear all data and re-load new one. Used by selecting
        new preset and add/edit/delete profiles events.
        Note, If you have methods to call related to `self.lctrl`,
        do so before calling `ClearAll()` method which deletes
        the pre-set references making the data no longer available.
        """
        if reset_cmbx:
            prst = sorted([os.path.splitext(x)[0] for x in
                          os.listdir(self.user_prst) if
                          os.path.splitext(x)[1] == '.json'])
            self.cmbx_prst.Clear()
            self.cmbx_prst.AppendItems(prst)
            self.cmbx_prst.SetSelection(0)

        # get column widths now before calling ClearAll()
        colw = [self.lctrl.GetColumnWidth(0),
                self.lctrl.GetColumnWidth(1),
                self.lctrl.GetColumnWidth(2),
                self.lctrl.GetColumnWidth(3),
                ]
        self.lctrl.ClearAll()
        self.txt_1cmd.SetValue("")
        self.txt_2cmd.SetValue("")
        self.pass_1_pre.SetValue("")
        self.pass_2_pre.SetValue("")

        if self.array:
            del self.array[0:6]

        self.set_listctrl(colw)
    # ----------------------------------------------------------------#

    def set_listctrl(self, colw):
        """
        Populates Presets list with JSON data files.
        See `presets_manager_utils.py`
        """
        self.lctrl.InsertColumn(0, _('Name'), width=colw[0])
        self.lctrl.InsertColumn(1, _('Description'), width=colw[1])
        self.lctrl.InsertColumn(2, _('Output Format'), width=colw[2])
        self.lctrl.InsertColumn(3, _('Supported Format List'), width=colw[3])

        path = os.path.join(f'{self.user_prst}',
                            f'{self.cmbx_prst.GetValue()}.json'
                            )
        collections = json_data(path)
        if collections == 'error':
            return
        try:
            index = 0
            for name in collections:
                index += 1
                rows = self.lctrl.InsertItem(index, name['Name'])
                self.lctrl.SetItem(rows, 0, name['Name'])
                self.lctrl.SetItem(rows, 1, name["Description"])
                self.lctrl.SetItem(rows, 2, name["Output_extension"])
                self.lctrl.SetItem(rows, 3, name["Supported_list"])

        except (TypeError, KeyError):
            wx.MessageBox(_('Preset not supported!\nFILE: "{}"'.format(path)),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return
    # ----------------------Event handler (callback)----------------------#

    def on_preset_selection(self, event):
        """
        Event when user select a preset
        in the presets combobox list.
        """
        self.reset_list()
        self.on_deselect(self, cleardata=False)
        self.parent.statusbar_msg(f'{self.cmbx_prst.GetValue()}', None)
    # ------------------------------------------------------------------#

    def on_deselect(self, event, cleardata=True):
        """
        Event when deselecting a line by clicking
        in an empty space in the control list
        """
        if cleardata:
            self.txt_1cmd.SetValue("")
            self.txt_2cmd.SetValue("")
            self.pass_1_pre.SetValue("")
            self.pass_2_pre.SetValue("")
            del self.array[0:8]  # delete all: [0],[1],[2],[3],[4],[5],[6],[7]
        self.btn_copyprofile.Disable()
        self.btn_delprofile.Disable()
        self.btn_editprofile.Disable()
        self.parent.statusbar_msg("", None)
    # ------------------------------------------------------------------#

    def on_select(self, event):  # lctrl
        """
        Event when selecting a profile in the lctrl,
        this update the request data of the objects.
        """
        path = os.path.join(f'{self.user_prst}',
                            f'{self.cmbx_prst.GetValue()}.json'
                            )
        collections = json_data(path)
        selected = event.GetText()  # event.GetText is a Name Profile
        self.txt_1cmd.SetValue("")
        self.txt_2cmd.SetValue("")
        self.pass_1_pre.SetValue("")
        self.pass_2_pre.SetValue("")
        self.btn_copyprofile.Enable()
        self.btn_delprofile.Enable()
        self.btn_editprofile.Enable()
        del self.array[0:8]  # delete all: [0],[1],[2],[3],[4],[5],[6],[7]

        try:
            for name in collections:
                if selected == name["Name"]:  # profile name
                    self.array.append(name["Name"])
                    self.array.append(name["Description"])
                    self.array.append(name["First_pass"])
                    self.array.append(name["Second_pass"])
                    self.array.append(name["Supported_list"])
                    self.array.append(name["Output_extension"])
                    self.array.append(name["Preinput_1"])
                    self.array.append(name["Preinput_2"])

        except KeyError as err:
            wx.MessageBox(_('json key error\n'
                            'FILE: "{0}"\n\n{1}'.format(path, err)),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return

        self.txt_1cmd.AppendText(f'{self.array[2]}')  # cmd1 text ctrl
        self.pass_1_pre.AppendText(f'{self.array[6]}')  # cmd1 text ctrl
        if self.array[3]:
            self.txt_2cmd.Enable()
            self.pass_2_pre.Enable()
            self.txt_2cmd.AppendText(f'{self.array[3]}')  # cmd2 text ctrl
            self.pass_2_pre.AppendText(f'{self.array[7]}')  # cmd1 text ctrl
        else:
            self.txt_2cmd.Disable()
            self.pass_2_pre.Disable()

        sel = f'{self.cmbx_prst.GetValue()} - {self.array[0]}'
        self.parent.statusbar_msg(sel, None)
    # ------------------------------------------------------------------#

    def preset_new(self, event):
        """
        Create new `*.json` empty preset
        """
        filename = None
        with wx.FileDialog(self, _("Enter name for new preset"),
                           defaultDir=self.user_prst,
                           wildcard="Videomass presets (*.json;)|*.json;",
                           style=wx.FD_SAVE
                           | wx.FD_OVERWRITE_PROMPT) as fileDialog:

            if fileDialog.ShowModal() == wx.ID_CANCEL:
                return
            filename = f"{fileDialog.GetPath()}.json"
            try:
                with open(filename, 'w', encoding='utf-8') as file:
                    file.write('[]')
            except IOError:
                wx.LogError(_("Cannot save current "
                            "data in file '{}'.").format(filename))
                return
        if filename:
            wx.MessageBox(_("'Successful!\n\n"
                            "A new empty preset has been created."),
                          "Videomass ", wx.ICON_INFORMATION, self)

            self.reset_list(True)
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def preset_del(self, event):
        """
        Remove selected preset moving to the `Removals` folder
        """
        filename = self.cmbx_prst.GetValue()
        if wx.MessageBox(_('Are you sure you want to remove "{}" preset?\n\n '
                           'It will be moved to the "Removals" subfolder '
                           'inside the presets folder.').format(filename),
                         _('Please confirm'), wx.ICON_QUESTION
                         | wx.CANCEL | wx.YES_NO, self) != wx.YES:
            return

        try:
            if not os.path.exists(os.path.join(self.user_prst, 'Removals')):
                os.mkdir(os.path.join(self.user_prst, 'Removals'))
        except OSError as err:
            wx.MessageBox(_("{}\n\nSorry, removal failed, cannot "
                            "continue..").format(err),
                          _('Videomass - Error!'), wx.ICON_ERROR, self
                          )
            return

        s = os.path.join(self.user_prst, f'{filename}.json')
        d = os.path.join(self.user_prst, 'Removals', f'{filename}.json')
        os.replace(s, d)

        wx.MessageBox(_('The preset "{0}" was successfully '
                        'removed').format(filename), _('Videomass - Error!'),
                      wx.ICON_ERROR, self
                      )
        self.reset_list(True)
        self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def preset_export(self, event):
        """
        save one preset on media
        """
        combvalue = self.cmbx_prst.GetValue()
        filedir = f'{self.user_prst}/{combvalue}.json'

        dlg = wx.DirDialog(self, _("Choose Destination"),
                           "", style=wx.DD_DEFAULT_STYLE)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            if os.path.exists(os.path.join(path, f'{combvalue}.json')):
                if wx.MessageBox(_('A file with this name already exists, '
                                   'do you want to overwrite it?'),
                                 _('Please confirm'), wx.ICON_QUESTION
                                 | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                    return

            status = copy_restore(filedir,
                                  os.path.join(path, f'{combvalue}.json'))
            dlg.Destroy()

            if status:
                wx.MessageBox(f'{status}', _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
                return
            wx.MessageBox(_("The preset was exported successfully"),
                          "Videomass", wx.OK, self)
    # ------------------------------------------------------------------#

    def preset_export_all(self, event):
        """
        Save the presets directory on media
        """
        src = self.user_prst

        dialsave = wx.DirDialog(self, _("Choose Destination"),
                                "", wx.DD_DEFAULT_STYLE)
        if dialsave.ShowModal() == wx.ID_OK:
            dest = dialsave.GetPath()
            status = copydir_recursively(src, dest, 'Videomass-Presets-copy')
            dialsave.Destroy()
            if status:
                wx.MessageBox(f'{status}', _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
            else:
                wx.MessageBox(_("All presets have been exported successfully"),
                              "Videomass", wx.OK, self)
    # ------------------------------------------------------------------#

    def preset_import(self, event):
        """
        Import a new preset. If the preset already exists you will
        be asked to overwrite it or not.
        """
        wildcard = "Source (*.json)|*.json| All files (*.*)|*.*"

        with wx.FileDialog(self, _("Import a new preset"),
                           "", "", wildcard, wx.FD_OPEN
                           | wx.FD_FILE_MUST_EXIST) as filedlg:

            if filedlg.ShowModal() == wx.ID_CANCEL:
                return

            newincoming = filedlg.GetPath()
            new = os.path.basename(newincoming)

        if not newincoming.endswith('.json'):
            wx.MessageBox(_('Error, invalid preset: "{}"').format(
                          os.path.basename(newincoming)),
                          _('Videomass - Error!'), wx.ICON_ERROR, self
                          )
            return

        if os.path.exists(os.path.join(self.user_prst, new)):

            if wx.MessageBox(_("This preset already exists and is about to be "
                               "updated. Don't worry, it will keep all your "
                               "saved profiles.\n\n"
                               "Do you want to continue?"),
                             _('Please confirm'), wx.ICON_QUESTION
                             | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                return

            update_oudated_profiles(newincoming,
                                    os.path.join(self.user_prst, new))
        status = copy_restore(newincoming, os.path.join(self.user_prst, new))
        if status:
            wx.MessageBox(f'{status}', _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        self.reset_list(True)  # reload presets
        self.on_deselect(self, cleardata=False)
        wx.MessageBox(_("A new preset was successfully imported"),
                      "Videomass", wx.OK, self)
    # ------------------------------------------------------------------#

    def preset_import_all(self, event):
        """
        This method depends on the event given as argument: If it is
        `None` it will restore the user's preset directory to the
        directory given by the `source` attribute. Otherwise the
        event will be triggered by clicking on the `Import group`
        button which will have a slightly different behavior. In any
        case it will not overwrite existing presets but will update
        them with missing profiles on the destination files.
        In addition it will copy all other presets that do not yet
        exist on the destination.
        """
        source = self.src_prst
        if event:
            if wx.MessageBox(_("This will update the presets database. "
                               "Don't worry, it will keep all your saved "
                               "profiles.\n\nDo you want to continue?"),
                             _("Please confirm"), wx.ICON_QUESTION
                             | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                return None

            dialsave = wx.DirDialog(self, _("Import a new presets folder"),
                                    "", style=wx.DD_DEFAULT_STYLE)
            if dialsave.ShowModal() == wx.ID_CANCEL:
                return None
            source = dialsave.GetPath()
            dialsave.Destroy()

        # create a dir backup
        datenow = time.strftime('%H%M%S-%a_%d_%B_%Y')
        err = copydir_recursively(self.user_prst, self.appdata['confdir'],
                                  f'presets-{datenow}-Backup')
        if err:
            wx.MessageBox(f'{err}', _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return err

        incom = [n for n in os.listdir(source) if n.endswith('.json')]
        outcom = [n for n in os.listdir(self.user_prst) if n.endswith('.json')]

        # Return a new set with elements common to the set and all others.
        # In short, copy only files with matching basenames.
        for f in set(incom).intersection(outcom):
            err = update_oudated_profiles(os.path.join(source, f),
                                          os.path.join(self.user_prst, f))
            if err:
                wx.MessageBox(f"{err}", _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
                return err
        # copies non-existent ones to the destination directory
        if event:  # only `Import group` event
            err = copy_on('prst', source, self.user_prst, overw=False)
            if err:
                wx.MessageBox(f"{err}", _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
                return err

        wx.MessageBox(_("The presets database has been successfully "
                        "updated"), "Videomass", wx.OK, self)
        self.reset_list(True)
        self.on_deselect(self, cleardata=False)
        return None
    # ------------------------------------------------------------------#

    def preset_default(self, event):
        """
        Replace the selected preset at default values.
        """
        msg = _("Be careful! The selected preset will be "
                "overwritten with the default one. Your profiles "
                "may be deleted!\n\nDo you want to continue?"
                )
        if wx.MessageBox(msg, _("Please confirm"),
                         wx.ICON_WARNING
                         | wx.YES_NO
                         | wx.CANCEL,
                         self) == wx.YES:

            filename = self.cmbx_prst.GetValue()
            status = copy_restore(f'{self.src_prst}/{filename}.json',
                                  f'{self.user_prst}/{filename}.json'
                                  )
            if status:
                wx.MessageBox(status, _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
                return

            wx.MessageBox(_("Successful recovery"), "Videomass", wx.OK, self)
            self.reset_list()  # reload presets
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def preset_default_all(self, event):
        """
        restore all preset files directory
        """
        if wx.MessageBox(_("Be careful! This action will restore all presets "
                           "to default ones. Your profiles may be deleted!\n\n"
                           "In any case, to avoid data loss, the presets "
                           "folder will be backed up in the program's "
                           "configuration directory."
                           "\n\nDo you want to continue?"),
                         _("Please confirm"),
                         wx.ICON_WARNING | wx.YES_NO | wx.CANCEL,
                         self) == wx.YES:

            if os.path.exists(self.user_prst):
                # create a dir backup
                datenow = time.strftime('%H%M%S-%a_%d_%B_%Y')
                err = os.rename(self.user_prst,
                                f"{self.user_prst}-{datenow}-Backup")
                if err:
                    wx.MessageBox(f'{err}', _('Videomass - Error!'),
                                  wx.ICON_ERROR, self)
                    return

            err = copydir_recursively(self.src_prst, self.appdata['confdir'])
            if err:
                wx.MessageBox(f"{err}", _('Videomass - Error!'),
                              wx.ICON_ERROR, self)
            else:
                wx.MessageBox(_("All default presets have been "
                                "successfully recovered"),
                              "Videomass", wx.OK, self)
                self.reset_list(True)
                self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def presets_refresh(self, event):
        """
        Force to to re-charging
        """
        self.reset_list(True)
        self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def profile_add(self, event):
        """
        Store new profiles in the selected preset
        """
        filename = self.cmbx_prst.GetValue()
        title = _('New Profile - Preset "{}"').format(filename)
        prstdialog = setting_profiles.SettingProfile(self,
                                                     'newprofile',
                                                     filename,
                                                     None,
                                                     title,
                                                     )
        ret = prstdialog.ShowModal()
        if ret == wx.ID_OK:
            self.reset_list()  # re-charging lctrl with newer
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def profile_edit(self, event):
        """
        Edit an existing profile
        """
        filename = self.cmbx_prst.GetValue()
        title = _('*Edit Profile - Preset "{}"').format(filename)
        prstdialog = setting_profiles.SettingProfile(self,
                                                     'edit',
                                                     filename,
                                                     self.array,
                                                     title)
        ret = prstdialog.ShowModal()
        if ret == wx.ID_OK:
            self.reset_list()  # re-charging lctrl with newer
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def profile_copy(self, event):
        """
        Copy (duplicate) selected profile
        """
        filename = os.path.join(f'{self.user_prst}',
                                f'{self.cmbx_prst.GetValue()}.json'
                                )
        newprst = write_new_profile(filename,
                                    Name=f'{self.array[0]} (duplicated)',
                                    Description=self.array[1],
                                    First_pass=self.array[2],
                                    Second_pass=self.array[3],
                                    Supported_list=self.array[4],
                                    Output_extension=self.array[5],
                                    Preinput_1=self.array[6],
                                    Preinput_2=self.array[7],
                                    )
        if not newprst:
            self.reset_list()
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def profile_del(self, event):
        """
        Delete a selected profile

        """
        if wx.MessageBox(_("Are you sure you want to delete the "
                           "selected profile? It will no longer be "
                           "possible to recover it."), _("Please confirm"),
                         wx.ICON_WARNING | wx.YES_NO | wx.CANCEL,
                         self) == wx.YES:

            filename = os.path.join(f'{self.user_prst}',
                                    f'{self.cmbx_prst.GetValue()}.json'
                                    )
            delete_profiles(filename, self.array[0])
            self.reset_list()
            self.on_deselect(self, cleardata=False)
    # ------------------------------------------------------------------#

    def check_options(self, index=None):
        """
        Update entries and file check.
        """
        if not self.array:
            self.parent.statusbar_msg(_("First select a profile in the list"),
                                      PrstPan.YELLOW, PrstPan.BLACK)
            return None

        if (self.array[2].strip() != self.txt_1cmd.GetValue().strip()
                or self.array[3].strip() != self.txt_2cmd.GetValue().strip()):
            if self.txtcmdedited:

                msg = _("The selected profile command has been "
                        "changed manually.\n"
                        "Do you want to apply it "
                        "during the conversion process?")
                dlg = wx.RichMessageDialog(self, msg,
                                           _("Please confirm"),
                                           wx.ICON_QUESTION
                                           | wx.CANCEL
                                           | wx.YES_NO,
                                           )
                dlg.ShowCheckBox(_("Don't show this dialog again"))

                if dlg.ShowModal() != wx.ID_YES:
                    if dlg.IsCheckBoxChecked():
                        # make sure we won't show it again the next time
                        self.txtcmdedited = False
                    return None
                if dlg.IsCheckBoxChecked():
                    # make sure we won't show it again the next time
                    self.txtcmdedited = False

        if index is not None:
            infile = [self.parent.file_src[index]]
            outfilenames = [self.parent.outputnames[index]]
        else:
            infile = self.parent.file_src
            outfilenames = self.parent.outputnames

        outext = '' if self.array[5] == 'copy' else self.array[5]
        extlst = self.array[4]
        src = supported_formats(extlst, infile)
        filecheck = check_files(src,
                                self.appdata['outputdir'],
                                self.appdata['outputdir_asinput'],
                                self.appdata['filesuffix'],
                                outext,
                                outfilenames,
                                )
        if not filecheck:
            # not supported, missing files or user has changed his mind
            return None

        return filecheck
    # ----------------------------------------------------------------#

    def get_codec_args(self):
        """
        Get data from encoder panels and set ffmpeg
        command arguments
        """
        if self.array[3]:
            thrtype = 'Two pass'
        else:
            thrtype = 'One pass'

        preinput_1 = " ".join(self.pass_1_pre.GetValue().split())
        preinput_2 = " ".join(self.pass_2_pre.GetValue().split())
        pass1 = " ".join(self.txt_1cmd.GetValue().split())
        pass2 = " ".join(self.txt_2cmd.GetValue().split())
        preset = self.cmbx_prst.GetValue()
        kwargs = {'type': thrtype, 'args': [pass1, pass2],
                  'pre-input-1': preinput_1, 'pre-input-2': preinput_2,
                  'preset name': f'Presets Manager - {preset}',
                  }
        return kwargs
    # ----------------------------------------------------------------#

    def queue_mode(self):
        """
        build queue mode arguments. This method is
        called by `parent.on_add_to_queue`.
        Return a dictionary of data.
        """
        logname = 'Queue Processing.log'
        index = self.parent.file_src.index(self.parent.filedropselected)

        check = self.check_options(index)
        if not check:
            return None

        f_src, f_dest = check[0][0], check[1][0]
        kwargs = self.get_codec_args()
        dur, ss, et = update_timeseq_duration(self.parent.time_seq,
                                              self.parent.duration
                                              )
        kwargs['start-time'], kwargs['end-time'] = ss, et
        kwargs['logname'] = logname
        kwargs['extension'] = '' if self.array[5] == 'copy' else self.array[5]
        kwargs['source'] = f_src
        kwargs['destination'] = f_dest
        kwargs["duration"] = dur[index]

        return kwargs
    # ------------------------------------------------------------------#

    def batch_mode(self):
        """
        build batch mode arguments. This method is called
        by `parent.click_start`
        """
        logname = 'Presets Manager.log'

        check = self.check_options()
        if not check:
            return None

        f_src, f_dest = check
        kwargs = self.get_codec_args()
        dur, ss, et = update_timeseq_duration(self.parent.time_seq,
                                              self.parent.duration
                                              )
        kwargs['start-time'], kwargs['end-time'] = ss, et
        kwargs['extension'] = '' if self.array[5] == 'copy' else self.array[5]

        batchlist = []
        for index in enumerate(self.parent.file_src):
            kw = kwargs.copy()
            kw['source'] = f_src[index[0]]
            kw['destination'] = f_dest[index[0]]
            kw['duration'] = dur[index[0]]
            batchlist.append(kw)

        keyval = self.update_dict(len(self.parent.file_src), **kwargs)
        ending = Formula(self, (700, 200),
                         self.parent.movetotrash,
                         self.parent.emptylist,
                         **keyval,
                         )
        if ending.ShowModal() == wx.ID_OK:
            (self.parent.movetotrash,
             self.parent.emptylist) = ending.getvalue()
            self.parent.switch_to_processing(kwargs["type"],
                                             logname,
                                             datalist=batchlist)
        return None
    # ----------------------------------------------------------------#

    def update_dict(self, cntmax, **kwa):
        """
        Update information before send to epilogue

        """
        passes = '2' if self.array[3] else '1'

        if not self.parent.time_seq:
            sst, endt = _('Same as source'), _('Same as source')
        else:
            sst = kwa["start-time"].split()[1]
            endt = kwa["end-time"].split()[1]

        if self.appdata['outputdir_asinput']:
            dest = _('Same destination paths as source files')
        else:
            dest = self.appdata['outputdir']

        keys = (_("Batch processing items\nDestination\nAutomation/Preset\n"
                  "Encoding passes\nProfile Used\nOutput Format\n"
                  "Start of segment\nClip duration"))
        vals = (f"{cntmax}\n{dest}\n{kwa['preset name']}"
                f"\n{passes}\n{self.array[0]}\n{self.array[5]}\n{sst}\n{endt}"
                )
        return {'key': keys, 'val': vals}



# -*- coding: UTF-8 -*-
"""
Name: widget_utils.py
Porpose: Features a set of useful wx widgets to use dynamically
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Nov.27.2021
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import wx
import wx.adv
from pubsub import pub


class NormalTransientPopup(wx.PopupTransientWindow):
    """
    Adds a bit of text and mouse movement to the wx.PopupWindow
    """
    def __init__(self, parent, style, message, backgrdcol, foregrdcol):
        wx.PopupTransientWindow.__init__(self, parent, style)
        panel = wx.Panel(self)
        panel.SetBackgroundColour(backgrdcol)
        st = wx.StaticText(panel, -1, message)
        st.SetForegroundColour(wx.Colour(foregrdcol))
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(st, 0, wx.ALL, 5)
        panel.SetSizer(sizer)

        sizer.Fit(panel)
        sizer.Fit(self)
        self.Layout()


def notification_area(title, message, flag, timeout=5):
    """
    Show the user a message on system tray (Notification Area)

    Various options can be set after the message is created if desired.
    notify.SetFlags(wx.ICON_INFORMATION/wx.ICON_WARNING/wx.ICON_ERROR)
    notify.SetTitle("Wooot")
    notify.SetMessage("It's a message!")
    notify.SetParent(self)
    """
    notify = wx.adv.NotificationMessage(title=title,
                                        message=message,
                                        parent=None,
                                        flags=flag
                                        )
    notify.Show(timeout=timeout)  # 1 for short timeout, 100 for long timeout
    # notify.Close()       # Hides the notification.


class PopupDialog(wx.Dialog):
    """
    A pop-up dialog box for temporary messages that tell the
    user the process in progress (usually short/medium duration
    operations).
    New in version 5.0.12:
        If your thread has the `stop` method you can pass a running
        `thread` as default arg to interface with the auto-displayed
        Stop button (See `VolumeDetectThread` class as example model).

    Usage:
            loadDlg = PopupDialog(parent, caption, message, thread)
            loadDlg.ShowModal()
            loadDlg.Destroy()
    """
    def __init__(self, parent=None,
                 caption='Pop-Up Dialog',
                 msg="Wait, I'm completing a task...",
                 thread=None,
                 ):
        # Create a dialog
        wx.Dialog.__init__(self, parent, -1, caption, size=(350, 150),
                           style=wx.CAPTION)
        # Add sizers
        self.thread = thread
        boxv = wx.BoxSizer(wx.VERTICAL)
        boxh = wx.BoxSizer(wx.HORIZONTAL)

        if hasattr(wx, 'ActivityIndicator'):  # wxPython => 4.1
            # activity add indicator
            ai = wx.ActivityIndicator(self)
            ai.Start()
            boxh.Add(ai, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL | wx.ALL, 10)
        # Add the message
        message = wx.StaticText(self, -1, msg, style=wx.ALIGN_CENTRE_VERTICAL)
        boxh.Add(message, 0, wx.EXPAND | wx.ALL, 10)
        boxv.Add(boxh, 0, wx.EXPAND)
        # Add an Info graphic
        bitmap = wx.Bitmap(48, 48)
        bitmap = wx.ArtProvider.GetBitmap(wx.ART_INFORMATION,
                                          wx.ART_MESSAGE_BOX, (48, 48)
                                          )
        graphic = wx.StaticBitmap(self, -1, bitmap)
        boxh.Add(graphic, 0, wx.TOP | wx.ALL, 10)

        if self.thread:
            # show button stop
            gridbtn = wx.GridSizer(1, 1, 0, 0)
            boxv.Add(gridbtn, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)
            button_stop = wx.Button(self, wx.ID_ANY, _("Stop"))
            gridbtn.Add(button_stop, 1, wx.ALL, 5)
            self.Bind(wx.EVT_BUTTON, self.on_stop, button_stop)

        # Handle layout
        self.SetAutoLayout(True)
        self.SetSizer(boxv)
        self.Fit()
        self.Layout()

        pub.subscribe(self.getMessage, "RESULT_EVT")
    # ----------------------------------------------------------#

    def on_stop(self, event):
        """
        If thread is not None, you can call the thread stop
        method here.
        """
        self.thread.stop()

    def getMessage(self, status):
        """
        Process report terminated. This method is called using
        pub/sub protocol (see generic_download.py, generic_task.py,
        volumedetect.py, ytd_extractinfo.py, filter_stab.py),
        it riceive msg and status from current thread.
        NOTE:
        All'inizio usavo self.Destroy() per chiudere il dialogo modale
        (con modeless ritornava dati None), ma dava warning e critical
        e su OsX non chiudeva affatto. Con EndModal ho risolto tutti
        i problemi e funziona bene. Ma devi ricordarti di eseguire
        Destroy() dopo ShowModal() nel chiamante.
        vedi: https://github.com/wxWidgets/Phoenix/issues/672
        Penso sia fattibile anche implementare un'interfaccia GetValue
        su questo dialogo, ma si perderebbe un po' di portabilità.

        """
        # self.Destroy() # do not work
        # self.ai.Stop()
        self.EndModal(1)


class CountDownDlg(wx.Dialog):
    """
    This dialog notifies the user that something will happen
    when the countdown expires, e.g. exiting the application
    or shutting down the system.
    At the end of the countdown this dialogue will
    self-destroy and events will be allowed to continue unless
    the user clicks on the cancel button.

    Usage:
        dlg = CountDownDlg(self,
                           timeout=10,
                           message='Something happens in {0} seconds',
                           caption='My awesome title'
                           )
        res = dlg.ShowModal() == wx.ID_OK
        dlg.Destroy()
        if res:
            ...other code here
    """
    get = wx.GetApp()  # get data from bootstrap
    APPICON = get.iconset['videomass']
    TIMER_INTERVAL = 1000  # milliseconds

    def __init__(self, parent, timeout, message, caption):
        """
        parent: -1 to make parent, use 'None' otherwise
        timeout: time second (int)
        message: a message including the format {0} (str)
        caption: title for caption (str)
        """
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)

        self.timeout = timeout
        self.message = message

        # ------ Add widget controls
        sizbase = wx.BoxSizer(wx.VERTICAL)
        sizbase.Add((0, 20), 0)
        self.msgtxt = wx.StaticText(self, wx.ID_ANY,
                                    self.message.format(self.timeout),
                                    style=wx.ALIGN_CENTRE_VERTICAL
                                    )
        sizbase.Add(self.msgtxt, 1, wx.ALL | wx.ALIGN_CENTRE, 5)
        # ------ bottom layout buttons
        sizbott = wx.BoxSizer(wx.HORIZONTAL)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        sizbott.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        sizbott.Add(btn_ok, 0, wx.LEFT, 5)
        sizbase.Add(sizbott, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        # ------ Properties
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(CountDownDlg.APPICON,
                                      wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetTitle(caption)
        self.SetMinSize((400, 150))
        self.SetSizer(sizbase)
        sizbase.Fit(self)
        self.Layout()
        self.Center()

        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.on_timer, self.timer)
        self.timer.Start(self.TIMER_INTERVAL)
    # ------------------------------------------------------------------#

    def autodestroy(self):
        """
        stop the timer and destroy this dialog.
        """
        self.timer.Stop()
        return self.Destroy()

    # ----------------------Event handler (callback)----------------------#

    def on_timer(self, event):
        """
        Set the timer countdown on message
        """
        self.timeout -= 1
        self.msgtxt.SetLabel(self.message.format(self.timeout))
        if self.timeout <= 0:
            self.EndModal(wx.ID_OK)



# -*- coding: UTF-8 -*-
"""
Name: edit_queue.py
Porpose: edit a selected item queue
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.03.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx


class Edit_Queue_Item(wx.Dialog):
    """
    This dialog is useful to edit command arguments on the fly
    before run the process.

    """
    PASS_1 = _("One-Pass, Do not start with `ffmpeg "
               "-i filename`; do not end with "
               "`output-filename`"
               )
    PASS_2 = _("Two-Pass (optional), Do not start with "
               "`ffmpeg -i filename`; do not end with "
               "`output-filename`"
               )
    # ------------------------------------------------------------------

    def __init__(self, parent, kwargs):
        """
        `kwargs` is a class dict
        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.kwargs = kwargs
        if self.appdata['ostype'] == 'Darwin':
            fontsize = 10
        else:
            fontsize = 8

        wx.Dialog.__init__(self, parent, -1,
                           _("Videomass - Edit the selected "
                             "item in the queue"),
                           style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        size_base = wx.BoxSizer(wx.VERTICAL)
        size_base.Add((0, 15), 0)
        labpass1 = wx.StaticText(self, wx.ID_ANY, Edit_Queue_Item.PASS_1)
        size_base.Add(labpass1, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_1_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE
                                      )
        size_base.Add(self.pass_1_cmd, 1, wx.ALL | wx.EXPAND, 5)

        msg = _("Pre-input arguments for One-Pass (optional)")
        labpre1 = wx.StaticText(self, wx.ID_ANY, msg)
        size_base.Add(labpre1, 0, wx.ALL | wx.EXPAND, 5)

        self.pass_1_pre = wx.TextCtrl(self, wx.ID_ANY, "")
        size_base.Add(self.pass_1_pre, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_1_cmd.SetFont(wx.Font(fontsize,
                                        wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL,
                                        wx.NORMAL
                                        ))
        self.pass_1_cmd.SetToolTip(_('FFmpeg arguments for one-pass encoding'))
        self.pass_1_pre.SetFont(wx.Font(fontsize,
                                        wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL,
                                        wx.NORMAL
                                        ))
        tip = (_('Any optional arguments to add before input file on the '
                 'one-pass encoding, e.g required names of some hardware '
                 'accelerations like -hwaccel to use with CUDA.'))
        self.pass_1_pre.SetToolTip(tip)
        self.pass_1_cmd.AppendText(self.kwargs['args'][0])  # command 1
        self.pass_1_pre.AppendText(self.kwargs['pre-input-1'])  # input 1
        # box
        if self.kwargs['args'][1].strip() != '':
            labpass2 = wx.StaticText(self, wx.ID_ANY, Edit_Queue_Item.PASS_2)
            size_base.Add(labpass2, 0, wx.ALL | wx.EXPAND, 5)
            self.pass_2_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                          style=wx.TE_MULTILINE
                                          )
            size_base.Add(self.pass_2_cmd, 1, wx.ALL | wx.EXPAND, 5)
            msg = _("Pre-input arguments for Two-Pass (optional)")
            labpre2 = wx.StaticText(self, wx.ID_ANY, msg)
            size_base.Add(labpre2, 0, wx.ALL | wx.EXPAND, 5)

            self.pass_2_pre = wx.TextCtrl(self, wx.ID_ANY, "")
            size_base.Add(self.pass_2_pre, 0, wx.ALL | wx.EXPAND, 5)
            self.pass_2_cmd.SetFont(wx.Font(fontsize,
                                            wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL,
                                            wx.NORMAL
                                            ))
            tip = _('FFmpeg arguments for two-pass encoding')
            self.pass_2_cmd.SetToolTip(tip)
            self.pass_2_pre.SetFont(wx.Font(fontsize,
                                            wx.FONTFAMILY_TELETYPE,
                                            wx.NORMAL,
                                            wx.NORMAL
                                            ))
            tip = (_('Any optional arguments to add before input file on the '
                     'two-pass encoding, e.g required names of some hardware '
                     'accelerations like -hwaccel to use with CUDA.'))
            self.pass_2_pre.SetToolTip(tip)
            self.pass_2_cmd.AppendText(self.kwargs['args'][1])
            self.pass_2_pre.AppendText(self.kwargs['pre-input-2'])
            minsize = (800, 550)
        else:
            minsize = (800, 275)

        # ----- confirm buttons section
        btncancel = wx.Button(self, wx.ID_CANCEL, "")
        btnok = wx.Button(self, wx.ID_OK)
        btngrid = wx.FlexGridSizer(1, 2, 0, 0)
        btngrid.Add(btncancel, 0)
        btngrid.Add(btnok, 0, wx.LEFT, 5)
        size_base.Add(btngrid, flag=wx.ALL
                      | wx.ALIGN_RIGHT
                      | wx.RIGHT, border=5
                      )
        # ------ Set Layout
        self.SetMinSize(minsize)
        self.SetSizer(size_base)
        self.Fit()
        self.Layout()

        # ----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_cancel, btncancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btnok)

    # ---------------------Callbacks (event handler)----------------------#

    def on_ok(self, event):
        """
        Accept and close this dialog.
        Don't use self.Destroy() here.
        """
        self.kwargs['args'][0] = self.pass_1_cmd.GetValue()
        self.kwargs['pre-input-1'] = self.pass_1_pre.GetValue()

        if self.kwargs['args'][1].strip() != '':
            self.kwargs['pre-input-2'] = self.pass_2_pre.GetValue()
            self.kwargs['args'][1] = self.pass_2_cmd.GetValue()

        event.Skip()
    # ------------------------------------------------------------------#

    def on_cancel(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
Name: queue_utils.py
Porpose: utils for queue managements
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.23.2024
Code checker: flake8, pylint .

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import json
import wx
from videomass.vdms_dialogs.queue_singlechoice import SingleChoice_Queue


def write_json_file_queue(data, queuefile=None):
    """
    Write queue json file
    """
    if not queuefile:
        get = wx.GetApp()
        appdata = get.appset
        queuefile = os.path.join(appdata["confdir"], 'queue.backup')
    with open(queuefile, 'w', encoding='utf-8') as outfile:
        json.dump(data, outfile, ensure_ascii=False, indent=4)
# --------------------------------------------------------------------


def load_json_file_queue(newincoming=None):
    """
    Locates, loads and validates a QUEUE json file.
    Note, a Videomass queue file cannot contain multiple
    occurrences of the 'destination' key value
    """
    if not newincoming:
        wild = "Source (*.json)|*.json| All files (*.*)|*.*"
        with wx.FileDialog(None, _("Import queue file"),
                           defaultDir=os.path.expanduser('~'),
                           wildcard=wild,
                           style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as fdlg:

            if fdlg.ShowModal() == wx.ID_CANCEL:
                return None
            newincoming = fdlg.GetPath()
    try:
        with open(newincoming, 'r', encoding='utf-8') as fln:
            newdata = json.load(fln)

    except json.decoder.JSONDecodeError as err:
        msg = _('You are attempting to load a json file written with '
                'invalid JSON encoding.')
        wx.MessageBox(f'{err}\nFILE: "{newincoming}"\n\n{msg}',
                      _('Videomass - Error!'), wx.STAY_ON_TOP
                      | wx.ICON_ERROR
                      | wx.OK,
                      None
                      )
        return None

    keys = ('type', 'args', 'extension', 'logname', 'source', 'preset name',
            'destination', 'duration', 'start-time', 'end-time',)
    msg = (_('ERROR: invalid data found loading queue file.\n'
             'FILE: "{0}"\n\nKeys mismatched for '
             'requested data.').format(newincoming))
    for ck in newdata:
        for key in keys:
            if key not in ck:
                wx.MessageBox(msg, _('Videomass - Error!'), wx.STAY_ON_TOP
                              | wx.ICON_ERROR
                              | wx.OK,
                              None
                              )
                return None
    occurences = []
    msg = (_('ERROR: invalid data found loading queue file.\n'
             'FILE: "{0}"\n\nCannot contain multiple occurrences '
             'in `destination` keys value.').format(newincoming))
    for item in newdata:
        occurences.append(item['destination'])
    if any(occurences.count(x) > 1 for x in occurences):
        wx.MessageBox(msg, _('Videomass - Error!'),
                      wx.STAY_ON_TOP | wx.ICON_ERROR | wx.OK, None)
        return None
    return newdata
# --------------------------------------------------------------------


def extend_data_queue(parent, currentqueue: list, newqueue: list) -> list:
    """
    This function is responsible for extending the items of
    the `currentqueue` list while maintaining the same id.
    The result varies based on the choice of a specific
    selection given by the `choice` object.
    """
    indx_orig = []
    indx_new = []
    for indx1, olditem in enumerate(currentqueue):
        for indx2, newitem in enumerate(newqueue):
            if olditem['destination'] == newitem['destination']:
                indx_orig.append(indx1)
                indx_new.append(indx2)

    if indx_orig and indx_new:
        choice = None
        with SingleChoice_Queue(parent) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                choice = dlg.getvalue()

        if choice == 0:
            indx_orig.sort(reverse=True)  # IMPORTANT before iterate
            for idx in indx_orig:
                del currentqueue[idx]
            currentqueue.extend(newqueue)
        elif choice == 1:
            indx_new.sort(reverse=True)  # IMPORTANT before iterate
            for idx in indx_new:
                del newqueue[idx]
            currentqueue.extend(newqueue)
        elif choice == 2:
            currentqueue.clear()
            currentqueue.extend(newqueue)
        else:
            return None
    else:
        currentqueue.extend(newqueue)

    return currentqueue
